package contabilidad.beans;

import contabilidad.entity.*;
import io.jmix.core.DataManager;
import io.jmix.core.FetchPlan;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Component("contabi_AsientoBean")
public class AsientoBean {

    @Autowired
    private DataManager dataManager;

    private Empresa empresaDefecto;

    public Empresa getEmpresaDefecto() throws Exception{
        return empresaDefecto;
    }

    public Ejercicio getEjercicioDefecto(Empresa e) throws Exception{
        Date d = new Date();
        return getEjercicioForDate(d, getEmpresaDefecto());
    }

    public Ejercicio getEjercicioForDate(Date d, Empresa empresa) throws Exception{
        String query = "select ej from contabi_Ejercicio ej where ej.empresa = :emp";
        List<Ejercicio> l = dataManager.load(Ejercicio.class).query(query).parameter("emp", empresa).list();
        for (int i = 0; i < l.size(); i++) {
            Ejercicio ej = l.get(i);
            if ((ej.getFechaInicio().getTime()>=d.getTime()) && (ej.getFechaFin().getTime() >= d.getTime())){
                return ej;
            }
        }
        throw new Exception("No se pudo hallar un ejercicio aplicable");
    }

    public Integer getNuevoNumAsiento(Ejercicio ej) throws Exception{
        String query = "select max(a.numAsiento) from contabi_Asiento a join a.ejercicio e where e.id = :ej";
        List<Integer> l = dataManager.loadValue(query, Integer.class)
                .parameter("ej", ej.getId()).list();


        if (l.size()==0){
            return 1;
        }else if(l.size()==1){
            if (l.get(0)!=null) {
                Integer i = l.get(0);
                return ++i;
            }else{
                return 1;
            }
        }else{
            throw new Exception("No se pudo determinar numero de asiento");
        }
    }

    public Asiento duplicarAsiento(Asiento a) throws Exception{
        Asiento a2 = new Asiento();
        a2.setId(UUID.randomUUID());
        Integer numAsiento = getNuevoNumAsiento(a.getEjercicio());
        a2.setNumAsiento(numAsiento);
        a2.setDescripcion(a.getDescripcion());
        a2.setEjercicio(a.getEjercicio());
        a2.setFechaRegistro(new Date());
        a2.setFecha(new Date());
        a2.setApuntes(new ArrayList());
        a2.setMovtosIrpf(new ArrayList());
        a2.setMovtosIva(new ArrayList());
        for (int i = 0; i < a.getApuntes().size(); i++) {
            Apunte ap = a.getApuntes().get(i);
            Apunte ap2 = new Apunte();
            ap2.setId(UUID.randomUUID());
            ap2.setAsiento(a2);
            ap2.setNumApunte(ap.getNumApunte());
            ap2.setAmpliacion(ap.getAmpliacion());
            ap2.setImporte(ap.getImporte());
            ap2.setCuentaContable(ap.getCuentaContable());
            ap2.setDebeHaber(ap.getDebeHaber());
            a2.getApuntes().add(ap2);
        }
        for (int i = 0; i < a.getMovtosIva().size(); i++) {
            MovtoIva miva = a.getMovtosIva().get(i);
            MovtoIva miva2 = new MovtoIva();
            miva2.setId(UUID.randomUUID());
            miva2.setImporteBase(miva.getImporteBase());
            miva2.setAsiento(a2);
            miva2.setCuota(miva.getCuota());
            miva2.setNombre(miva.getNombre());
            miva2.setFechaDeclIva(miva.getFechaDeclIva());
            miva2.setTipoPorcentaje(miva.getTipoPorcentaje());
            miva2.setNumFra(miva.getNumFra());
            miva2.setNif(miva.getNif());
            a2.getMovtosIva().add(miva2);
        }
        for (int i = 0; i < a.getMovtosIrpf().size(); i++) {
            MovtoIrpf m = a.getMovtosIrpf().get(i);
            MovtoIrpf m2 = new MovtoIrpf();
            m2.setId(UUID.randomUUID());
            m2.setAsiento(a2);
            m2.setBase(m.getBase());
            m2.setTipoPorcentaje(m.getTipoPorcentaje());
            m2.setCuota(m.getCuota());
            m2.setFecha(m.getFecha());
            m2.setNif(m.getNif());
            m2.setNombre(m.getNombre());
            m2.setNumFra(m.getNumFra());
            a2.getMovtosIrpf().add(m2);
        }
        return a2;
    }


    public boolean realizaEliminacionAsiento(Asiento a) throws Exception{

        //borrado de asiento
        List entities = new ArrayList();


        List<Apunte> apuntes = new ArrayList(a.getApuntes());
        a.getApuntes().clear();


        List<MovtoIva> movtosIva = new ArrayList(a.getMovtosIva());
        a.getMovtosIva().clear();


        List<MovtoIrpf> movtosIrpf = new ArrayList(a.getMovtosIrpf());
        a.getMovtosIrpf().clear();


        List<PointerArchivoAdjuntoExtDoc> pointers = new ArrayList(a.getPointersArchivosAdjuntosExtDoc());
        a.getPointersArchivosAdjuntosExtDoc().clear();


        List<ImputacionCentroDeCoste> imputaciones = new ArrayList(a.getImputacionesCentroDeCoste());
        a.getImputacionesCentroDeCoste().clear();

        entities.addAll(apuntes);
        entities.addAll(movtosIrpf);
        entities.addAll(movtosIva);
        entities.addAll(pointers);
        entities.addAll(imputaciones);
        entities.add(a);


        dataManager.remove(entities);

        return true;
    }

}