package contabilidad.beans;

import contabilidad.entity.*;
import io.jmix.core.DataManager;
import io.jmix.core.FetchPlan;
import io.jmix.core.FluentValuesLoader;
import io.jmix.core.entity.KeyValueEntity;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

@Component("contabi_SaldosBean")
public class SaldosBean {

    @Autowired
    private DataManager dataManager;

    public List<SaldoCuentaEjercicio> calculaSaldosCuentaEjercicioPara(Ejercicio ej) throws Exception{
        PlanContable pc = ej.getPlanContable();
        String eql = "select ej from contabi_Ejercicio ej join ej.planContable pc where pc.id = :pcid and ej.anno = :anno";
        Ejercicio ejAnterior = dataManager.load(Ejercicio.class).query(eql)
                .parameter("pcid", pc.getId()).parameter("anno", ej.getAnno()-1).one();
        List<SaldoCuentaEjercicio> ssccee = dataManager.load(SaldoCuentaEjercicio.class)
                .query("select sce from contabi_SaldoCuentaEjercicio sce join sce.ejercicio ej where ej.id = :ejid")
                .parameter("ejid", ejAnterior.getId()).list();
        Hashtable<CuentaContable, SaldoCuentaEjercicio> ht = new Hashtable<>();
        for (int i = 0; i < ssccee.size(); i++) {
            SaldoCuentaEjercicio sce = ssccee.get(i);
            ht.put(sce.getCuentaContable(), sce);
        }
        List<CuentaContable> ccc = dataManager.load(CuentaContable.class).query("select cc from contabi_CuentaContable cc join cc.planContable pc where pc.id = :pcid")
                .parameter("pcid", pc.getId()).list();
        //creando un nuevo list de SaldoCuentaEjercicio para el ejercicio ej
        ArrayList<SaldoCuentaEjercicio> al = new ArrayList();
        for (int i = 0; i < ccc.size(); i++) {
            CuentaContable cc = ccc.get(i);
            SaldoCuentaEjercicio sce = dataManager.create(SaldoCuentaEjercicio.class);
            //para que los valores iniciales no sean nulos
            sce.setSaldoInicialDeudor(0.0);
            sce.setSaldoInicialAcreedor(0.0);

            sce.setEjercicio(ej);
            sce.setCuentaContable(cc);

            Double saldoAcreedor = 0.0;
            Double saldoDeudor = 0.0;
            if (ht.containsKey(cc)){
                saldoAcreedor = ht.get(cc).getSaldoInicialAcreedor();
                saldoDeudor = ht.get(cc).getSaldoInicialDeudor();
            }
            //sumando movtos debe haber del ejercicio anterior
            List<Apunte> aps = dataManager.load(Apunte.class).query("select ap from contabi_Apunte ap " +
                    "join ap.cuentaContable cc join ap.asiento a join a.ejercicio ej " +
                    "where ej.id = :ejid and cc.id = :ccid ")
                    .parameter("ejid", ejAnterior.getId())
                    .parameter("ccid", cc.getId()).list();
            double adDebe = 0.0;
            double adHaber = 0.0;
            for (int j = 0; j < aps.size(); j++) {
                Apunte ap = aps.get(j);
                if(ap.getDebeHaber()){
                    adDebe += ap.getImporte();
                }else{
                    adHaber += ap.getImporte();
                }
            }
            saldoAcreedor += adDebe;
            saldoDeudor += adHaber;

            if (saldoAcreedor >= saldoDeudor){
                sce.setSaldoInicialAcreedor(saldoAcreedor-saldoDeudor);
            }else{
                sce.setSaldoInicialDeudor(-saldoAcreedor+saldoDeudor);
            }

            al.add(sce);
        }
        return al;

    }

    public SaldoCuentaEjercicio getSaldoCuentaEjercicio(int anno, CuentaContable cc) throws Exception{
        Ejercicio ej = getEjercicioParaAnnoYEmpresa(cc.getEmpresa(), anno);
        String eql = "select sce from contabi_SaldoCuentaEjercicio sce join sce.ejercicio ej join sce.cuentaContable cc " +
                "where ej.id = :ejid and cc.codigo = :codigo";
        List<SaldoCuentaEjercicio> ss = dataManager.load(SaldoCuentaEjercicio.class).query(eql)
                .parameter("ejid", ej.getId())
                .parameter("codigo", cc.getCodigo())
                .list();
        if (ss.size()==0){
            return null;
        }
        if (ss.size()!=1){
            throw new Exception("Error al cargar saldo inicial de ejercicio " + anno + " para cuenta " + cc.getCodigo() + " de empresa " + cc.getEmpresa().getNombre());
        }else{
            try{
                SaldoCuentaEjercicio sce = ss.get(0);
                return sce;
            }catch(Exception exc){
                throw new Exception("Error al cargar saldo inicial de ejercicio " + anno + " para cuenta " + cc.getCodigo() + " de empresa " + cc.getEmpresa().getNombre());
            }
        }
    }

    private Ejercicio getEjercicioParaAnnoYEmpresa(Empresa emp, int anno) throws Exception{
        String eql = "select ej from contabi_Ejercicio ej join ej.empresa emp where ej.anno = :anno and emp.id = :empid";
        Ejercicio ej = dataManager.load(Ejercicio.class).query(eql)
                .parameter("anno", anno)
                .parameter("empid", emp.getId())
                .one();
        return ej;
    }


    /**
     * Devuelve array, primer elemento saldo debe, segundo saldo haber
     * @param cc
     * @param fecha
     * @return
     * @throws Exception
     */
    public double[] getSaldosCuentaAFecha(CuentaContable cc, Date fecha) throws Exception{
        int ejercicioAfectado = new Integer(new SimpleDateFormat("yyyy", Locale.getDefault()).format(fecha));
        SaldoCuentaEjercicio sce = getSaldoCuentaEjercicio(ejercicioAfectado, cc);

        double saldoInicialDeudor = 0.0;
        double saldoInicialAcreedor = 0.0;
        if (sce!=null){
            saldoInicialDeudor = sce.getSaldoInicialDeudor();
            saldoInicialAcreedor = sce.getSaldoInicialAcreedor();
        }
        Ejercicio ej = getEjercicioParaAnnoYEmpresa(cc.getEmpresa(), ejercicioAfectado);
        double totalDebe = getSumaParaCuentaAFecha(cc, ej.getFechaInicio(), fecha, true);
        double totalHaber = getSumaParaCuentaAFecha(cc, ej.getFechaInicio(), fecha, false);
        return new double[]{saldoInicialAcreedor + totalDebe, saldoInicialDeudor + totalHaber};
    }



    private double getSumaParaCuentaAFecha(CuentaContable cc, Date fechaDesde, Date fechaHasta, boolean debeHaber) throws Exception{
        PlanContable pc = cc.getPlanContable();
        String eql = "select sum(ap.importe) from contabi_Apunte ap join ap.cuentaContable cc join cc.planContable pc join ap.asiento a " +
                "where cc.id = :ccid and pc.id = :pcid and ap.debeHaber = :dh and a.fecha >= :fechadesde and a.fecha <= :fechahasta";

        double total = 0.0;
        try {
            List<Double> l = dataManager.loadValue(eql, Double.class)
                    .parameter("pcid", pc.getId())
                    .parameter("dh", debeHaber)
                    .parameter("fechadesde", fechaDesde)
                    .parameter("fechahasta", fechaHasta)
                    .parameter("ccid", cc.getId()).list();
            if (l.size()==1){
                try {
                    total = l.get(0);
                }catch(Exception exc){
                    total = 0.0;
                }
            }else if(l.size()==0){
                total = 0.0;
            }else{
                throw new Exception("Error al calcular saldo para cuenta " + cc.getCodigo() + " [SaldosBean]");
            }
        }catch(Exception exc){
            throw exc;
        }
        return total;
    }

    public List<Apunte> getApuntes(String[] codigosCuentas, Date deFecha, Date aFecha, Empresa emp){
        String eql = "select ap from contabi_Apunte ap join ap.asiento a join a.ejercicio ej join ej.empresa emp " +
                "where ap.cuentaContable.codigo in :ctas " +
                " and a.fecha >= :defecha and a.fecha <= :afecha and emp.id = :empid";

        String pam = "";
        for (int i = 0; i < codigosCuentas.length; i++) {
            if (i > 0) pam += ", ";
            pam += "'"+codigosCuentas[i]+"'";
        }

        List<Apunte> aps = dataManager.load(Apunte.class).query(eql)
                .fetchPlan(fpb->fpb.addFetchPlan(FetchPlan.BASE).add("cuentaContable", FetchPlan.BASE))
                .parameter("ctas",Arrays.asList(codigosCuentas))
                .parameter("defecha",deFecha)
                .parameter("afecha", aFecha)
                .parameter("empid", emp.getId())
                .list();
        return aps;
    }

    public List<Apunte> getApuntes(String deCodigoCuenta, String aCodigoCuenta, Date deFecha, Date aFecha, Empresa emp){
        List<Apunte> aps = dataManager.load(Apunte.class)
                .query("select ap from contabi_Apunte ap join ap.asiento a join a.ejercicio ej join ej.empresa emp " +
                        "where ap.cuentaContable.codigo >= :codigode " +
                        "and ap.cuentaContable.codigo <= :codigoa" +
                        " and a.fecha >= :defecha and a.fecha <= :afecha and emp.id = :empid")
                .fetchPlan(fpb->fpb.addFetchPlan(FetchPlan.BASE).add("cuentaContable", FetchPlan.BASE))
                .parameter("codigode",deCodigoCuenta)
                .parameter("codigoa", aCodigoCuenta)
                .parameter("defecha",deFecha)
                .parameter("afecha", aFecha)
                .parameter("empid", emp.getId())
                .list();
        return aps;
    }

    public List<PartidaBalance> transformaBalance(Hashtable<CuentaContable, Double[]> balance) throws Exception{
        Iterator<CuentaContable> cuentas = balance.keySet().iterator();
        ArrayList<PartidaBalance> al = new ArrayList();
        while(cuentas.hasNext()){
            CuentaContable cc = cuentas.next();
            Double[] saldos = balance.get(cc);
            PartidaBalance pb = new PartidaBalance();
            pb.setId(UUID.randomUUID());
            pb.setSaldo(saldos[2]);
            pb.setSaldoDebe(saldos[0]);
            pb.setSaldoHaber(saldos[1]);
            pb.setCodigoCuenta(cc.getCodigo());
            pb.setNombreCuenta(cc.getNombre());
            al.add(pb);
        }
        Collections.sort(al, new Comparator<PartidaBalance>(){
            public int compare(PartidaBalance pb1, PartidaBalance pb2){
                return pb1.getCodigoCuenta().compareTo(pb2.getCodigoCuenta());
            }
        });
        return al;
    }

    public Hashtable<CuentaContable, Double[]> getBalanceConAjustesDeCierre(Hashtable<CuentaContable, Double[]> balance) throws Exception{

        //regularizacion de perdidas y ganancias
        double acc6 = 0.0;
        double acc7 = 0.0;
        ArrayList<CuentaContable> ccs = new ArrayList(balance.keySet());
        CuentaContable cc129 = null;
        for (int i = 0; i < ccs.size(); i++) {
            CuentaContable cc = ccs.get(i);
            if (cc.getCodigo().substring(0,3).compareTo("129")==0){
                cc129 = cc;
            }
            Double[] saldos = balance.get(cc);
            if (cc.getCodigo().substring(0,1).compareTo("6")==0){
                if (saldos[0]>saldos[1]){
                    double ajuste = saldos[0]-saldos[1];
                    saldos[1] += ajuste;
                    saldos[2] = saldos[0] - saldos[1];
                    acc6 += ajuste;
                }else{
                    double ajuste = saldos[1]-saldos[0];
                    saldos[0] += ajuste;
                    saldos[2] = saldos[1] - saldos[0];
                    acc6 -= ajuste;
                }
            }
            if (cc.getCodigo().substring(0,1).compareTo("7")==0){
                if (saldos[1]>saldos[0]){
                    double ajuste = saldos[1]-saldos[0];
                    saldos[0] += ajuste;
                    saldos[2] = saldos[1] - saldos[0];
                    acc7 += ajuste;
                }else{
                    double ajuste = saldos[0]-saldos[1];
                    saldos[1] += ajuste;
                    saldos[2] = saldos[1] - saldos[0];
                    acc7 -= ajuste;
                }
            }

        }
        Double[] saldos129 = balance.get(cc129);
        saldos129[0] += acc6;
        saldos129[1] += acc7;
        saldos129[2] = saldos129[0]-saldos129[1];

        return balance;

    }

    public Double[] getSumaDebeHaberDeBalance(Hashtable<CuentaContable, Double[]> balance) throws Exception{
        Iterator<CuentaContable>  ccs = balance.keySet().iterator();
        double totalDebe = 0.0;
        double totalHaber = 0.0;
        while(ccs.hasNext()){
            CuentaContable cc = ccs.next();
            Double[] saldos = balance.get(cc);
            totalDebe += saldos[0];
            totalHaber += saldos[1];
        }
        return new Double[]{totalDebe, totalHaber};
    }

//    private static double round(double value, int places) {
//        if (places < 0) throw new IllegalArgumentException();
//
//        BigDecimal bd = new BigDecimal(Double.toString(value));
//        bd = bd.setScale(places, RoundingMode.DOWN);
//        return bd.doubleValue();
//    }

    public Hashtable<CuentaContable, Double[]> getBalanceAFecha(Date fecha, Ejercicio ej, PlanContable pc, int profundidad, boolean balanceSoloMovimientos) throws Exception{
        Date fechaDesde = ej.getFechaInicio();
        String eql = "select cc, sum(ap.importe) from contabi_Apunte ap " +
                "join ap.cuentaContable cc join cc.planContable pc join ap.asiento a " +
                "where pc.id = :pcid and a.fecha <= :fecha and a.fecha >= :defecha and ap.debeHaber = :dh group by cc";
        List<KeyValueEntity> saldosDebe = dataManager.loadValues(eql)
                .properties("cc", "sum")
                .parameter("pcid", pc.getId())
                .parameter("fecha", fecha)
                .parameter("defecha", fechaDesde)
                .parameter("dh", true).list();
        List<KeyValueEntity> saldosHaber = dataManager.loadValues(eql)
                .properties("cc", "sum")
                .parameter("pcid", pc.getId())
                .parameter("fecha", fecha)
                .parameter("defecha", fechaDesde)
                .parameter("dh", false).list();

        Hashtable<CuentaContable, Double[]> balance = new Hashtable<CuentaContable, Double[]>();
        for (int i = 0; i < saldosDebe.size(); i++) {
            KeyValueEntity kve = saldosDebe.get(i);
            CuentaContable cc = kve.getValue("cc");
            double totalDebe = kve.getValue("sum");
            Double[] saldos;
            if (!balance.containsKey(cc)){
                saldos = new Double[]{0.0,0.0,0.0};
                balance.put(cc, saldos);
            }
            saldos = balance.get(cc);
            saldos[0] = totalDebe;

        }
        for (int i = 0; i < saldosHaber.size(); i++) {
            KeyValueEntity kve = saldosHaber.get(i);
            CuentaContable cc = kve.getValue("cc");
            double totalDebe = kve.getValue("sum");
            Double[] saldos;
            if (!balance.containsKey(cc)){
                saldos = new Double[]{0.0,0.0,0.0};
                balance.put(cc, saldos);
            }
            saldos = balance.get(cc);
            saldos[1] = totalDebe;

        }
        Iterator<CuentaContable> cuentas = balance.keySet().iterator();
        while (cuentas.hasNext()) {
            CuentaContable cc = cuentas.next();
            Double[] saldos = balance.get(cc);
            saldos[2] = (saldos[0] - saldos[1]);
        }

        if (!balanceSoloMovimientos) {
            cuentas = balance.keySet().iterator();
            while (cuentas.hasNext()) {
                CuentaContable cc = cuentas.next();
                cc = dataManager.load(CuentaContable.class).id(cc.getId()).fetchPlan("cuentaContable-base-ext").one();
                Double[] saldos = balance.get(cc);
                SaldoCuentaEjercicio sce = getSaldoCuentaEjercicio(ej.getAnno(), cc);
                saldos[0] += sce.getSaldoInicialAcreedor();
                saldos[1] += sce.getSaldoInicialDeudor();
                saldos[2] = (saldos[0] - saldos[1]);

            }

            //completando el balance con las cuentas que NO han tenido movimientos en el ejercicio y por tanto
            //no han sido incluidos en el select anterior, pero que deben figurar en balance
            ArrayList cuentasIncluidas = new ArrayList(balance.keySet());
            String notInCuentas = "(";
            for (int i = 0; i < cuentasIncluidas.size(); i++) {
                CuentaContable cc = (CuentaContable) cuentasIncluidas.get(i);
                if (i > 0) notInCuentas += ", ";
                notInCuentas += "'" + cc.getCodigo() + "'";
            }
            notInCuentas += ")";
            eql = "select sce from contabi_SaldoCuentaEjercicio sce join sce.cuentaContable cc " +
                    "join sce.ejercicio ej where ej.id = :ejid and cc.codigo not in " + notInCuentas;
            List<SaldoCuentaEjercicio> l = dataManager.load(SaldoCuentaEjercicio.class).query(eql).parameter("ejid", ej.getId())
                    .list();
            for (int i = 0; i < l.size(); i++) {
                SaldoCuentaEjercicio sce = l.get(i);
                Double[] saldos = new Double[]{sce.getSaldoInicialAcreedor(), sce.getSaldoInicialDeudor(), sce.getSaldoInicialAcreedor() - sce.getSaldoInicialDeudor()};
                balance.put(sce.getCuentaContable(), saldos);
            }
        }




        if (profundidad == pc.getProfundidad().intValue()){
            return balance;
        }else{
            //recalculo de balance a profunidad inferior
            //involucra la creacion de cuentas contables virtuales
            Hashtable<String, Double[]> _balanceAjustado = new Hashtable<String, Double[]>();
            cuentas = balance.keySet().iterator();
            while(cuentas.hasNext()){
                CuentaContable cc = cuentas.next();
                Double[] saldos = balance.get(cc);
                saldos[2] = saldos[0] - saldos[1];
                String codigoAjustado = cc.getCodigo().substring(0,profundidad);
                if (!_balanceAjustado.containsKey(codigoAjustado)){
                    Double[] d = new Double[]{0.0,0.0,0.0};
                    _balanceAjustado.put(codigoAjustado, d);
                }
                Double[] d = _balanceAjustado.get(codigoAjustado);
                d[0] += saldos[0];
                d[1] += saldos[1];
                d[2] += saldos[2];
            }
            Hashtable<CuentaContable, Double[]> balanceAjustado = new Hashtable<CuentaContable, Double[]>();
            Iterator<String> cuentasStr = _balanceAjustado.keySet().iterator();
            while(cuentasStr.hasNext()){
                String cta = cuentasStr.next();
                String cta_comp = cta + StringUtils.repeat("0", pc.getProfundidad()-cta.length());
                CuentaContable cc;
                try {
                    cc = getCuentaContableDesdeCodigoYPlanContable(cta_comp, pc);
                }catch(Exception exc){
                    cc = new CuentaContable();
                    cc.setCodigo(cta);
                    cc.setNombre("CTA.AJ.NO HALLADA");
                    cc.setDescripcion(cc.getNombre());
                    cc.setId(UUID.randomUUID());

                }
                balanceAjustado.put(cc, _balanceAjustado.get(cta));
            }
            return balanceAjustado;
        }
    }

    public Hashtable<CuentaContable, Double[]> getBalanceAFecha(Date fecha, Ejercicio ej, PlanContable pc, int profundidad) throws Exception{
        return getBalanceAFecha(fecha, ej, pc, profundidad, false);
    }

    public Hashtable<CuentaContable, Double[]> eliminaCuentasSaldoCero(Hashtable<CuentaContable, Double[]> balance) throws Exception{
        Iterator<CuentaContable> it = balance.keySet().iterator();
        ArrayList<CuentaContable> toremove = new ArrayList<>();
        while(it.hasNext()){
            CuentaContable cc = it.next();
            Double[] saldos = balance.get(cc);
            if (saldos[2]<0.001){
                if ((saldos[0]<0.001) && (saldos[1]<0.001)){
                    toremove.add(cc);
                }

            }
        }
        for (int i = 0; i < toremove.size(); i++) {
            CuentaContable cc = toremove.get(i);
            balance.remove(cc);
        }
        return balance;
    }

    public boolean realizaEliminacionDeSaldosCuentaEjercicioParaEjercicio(Ejercicio ej) throws Exception{
        String eql = "select sce from contabi_SaldoCuentaEjercicio sce join sce.ejercicio ej where ej.id = :ejid";
        List<SaldoCuentaEjercicio> sces = dataManager.load(SaldoCuentaEjercicio.class).query(eql).parameter("ejid",ej.getId())
                .list();
        for (int i = 0; i < sces.size(); i++) {
            SaldoCuentaEjercicio sce = sces.get(i);
            dataManager.remove(sce);
        }
        return true;
    }

    public CuentaContable getCuentaContableDesdeCodigoYPlanContable(String codigo, PlanContable pc) throws Exception{
        String eql = "select cc from contabi_CuentaContable cc join cc.planContable pc where pc.id = :pcid and " +
                "cc.codigo like :codigo ";

        List<CuentaContable> cc = dataManager.load(CuentaContable.class).query(eql)
                .parameter("pcid", pc.getId())
                .parameter("codigo", codigo)
                .list();
        if (cc.size()==1){
            return cc.get(0);
        }else{
            throw new Exception("Error al seleccionar cuenta para datos:" + codigo + ", PlanContable:" + pc.getNombre());
        }
    }

    public List<CuentaContable> devuelveListaDeCuentasDesdeHasta(CuentaContable ccDesde, CuentaContable ccHasta) throws Exception{
        PlanContable pc = ccDesde.getPlanContable();
        String eql = "select cc from contabi_CuentaContable cc join cc.planContable pc " +
                "where cc.codigo >= :ccdesde and cc.codigo <= :cchasta and pc.id = :pcid";
        List<CuentaContable> l = dataManager.load(CuentaContable.class).query(eql)
                .parameter("ccdesde", ccDesde.getCodigo()).parameter("cchasta", ccHasta.getCodigo())
                .parameter("pcid", pc.getId())
                .list();
        return l;
    }

}