package contabilidad.beans;

import contabilidad.entity.Apunte;
import contabilidad.entity.CuentaContable;
import liquibase.pro.packaged.D;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@Component("contabi_TxaReportHelper")
public class Txarh {

    public static final int HORIZONTAL_ALIGNMENT_LEFT = -1;
    public static final int HORIZONTAL_ALIGNMENT_CENTER = 0;
    public static final int HORIZONTAL_ALIGNMENT_RIGHT = 1;

    DateFormat ddMMyyyy = new SimpleDateFormat("dd/MM/yyyy");
    DecimalFormat float2dp = new DecimalFormat("#,##0.00");

    public String getTxaReportHeaderForAccount(CuentaContable cc) throws Exception{
        String s = cc.getCodigo() + " " + cc.getNombre();
        return s;
    }

    public String getTxaReporForExtractoSimple(double[] saldos, List<Apunte> aps) throws Exception{

        Collections.sort(aps, comparatorAps);
        double saldoDeudor = saldos[0];
        double saldoAcreedor = saldos[1];
        String s = "";
        for (int i = 0; i < aps.size(); i++) {
            Apunte ap = aps.get(i);
            s += fillNColsFieldWithValue(ddMMyyyy.format(ap.getAsiento().getFecha()), 18);
            s += fillNColsFieldWithValue(ap.getAsiento().getNumAsiento() + "/" + "nappte.", 20);
            s += fillNColsFieldWithValue(ap.getAmpliacion(), 40);
            if (ap.getDebeHaber()){
                saldoDeudor += ap.getImporte();
                s += fillNColsFieldWithValue(" ", 5);
                s += fillNColsFieldWithValue(float2dp.format(ap.getImporte()), 15);
                s += "   a   ";
                s += fillNColsFieldWithValue("", 15);
            }else{
                saldoAcreedor += ap.getImporte();
                s += fillNColsFieldWithValue(" ", 5);
                s += fillNColsFieldWithValue("", 15);
                s += "   a   ";
                s += fillNColsFieldWithValue(float2dp.format(ap.getImporte()), 15);
            }
            //saldo
            s += fillNColsFieldWithValue(" " + float2dp.format(saldoDeudor), 25);
            s += fillNColsFieldWithValue(" / ", 5);
            s+= fillNColsFieldWithValue(" " + float2dp.format(saldoAcreedor), 25);
            s += breakLine();

        }

        return s;
    }

    public Comparator comparatorCC = new Comparator<CuentaContable>(){

        @Override
        public int compare(CuentaContable o1, CuentaContable o2) {
            return o1.getCodigo().compareTo(o2.getCodigo());
        }
    };

    public Comparator comparatorAps = new Comparator<Apunte>() {
        public int compare(Apunte ap1, Apunte ap2){
            int diff = ap1.getAsiento().getFecha().compareTo(ap2.getAsiento().getFecha());
            if (diff!=0){
                return diff;
            }else{
                if (ap1.getAsiento().getNumAsiento().compareTo(ap2.getAsiento().getNumAsiento())==0){
                    return ap1.getNumApunte().compareTo(ap2.getNumApunte());
                }else{
                    return ap1.getAsiento().getNumAsiento().compareTo(ap2.getAsiento().getNumAsiento());
                }

            }

        }
    };

    public String getTxaReportForBalanceSituacion (Hashtable<CuentaContable, Double[]> balance) throws Exception{



        String s = "";

        DecimalFormat df = new DecimalFormat("#,##0.00");
        ArrayList<CuentaContable> cuentas = new ArrayList(balance.keySet());
        Collections.sort(cuentas, comparatorCC);

        for (int i = 0; i < cuentas.size(); i++) {
            CuentaContable cc = cuentas.get(i);
            Double[] saldos = balance.get(cc);
            if (saldos[2]==0.0) continue;
            s += fillNColsFieldWithValue(cc.getCodigo(), 15);
            s += fillNColsFieldWithValue(cc.getNombre(), 40);
            s += fillNColsFieldWithValue(df.format(saldos[0]), 15);
            s += fillNColsFieldWithValue(df.format(saldos[1]), 15);
            s += fillNColsFieldWithValue(df.format(Math.abs(saldos[2])), 15);
            if (saldos[0]>saldos[1]){
                s += fillNColsFieldWithValue("D", 3);
            }else if (saldos[1]>saldos[0]){
                s += fillNColsFieldWithValue("H", 3);
            }
            s += breakLine();
        }


        return s;
    }

    public String breakLine(){
        return "\n";
    }


    public String fillNColsFieldWithValue(String value, int ncols, int horizontalAlignment) throws Exception{
        if (value.length()>=ncols){
            return value.substring(0,ncols);
        }else{
            if (horizontalAlignment==HORIZONTAL_ALIGNMENT_LEFT){
                return value + StringUtils.repeat(" ", ncols - value.length());
            }
            if (horizontalAlignment==HORIZONTAL_ALIGNMENT_CENTER){
                String prov =  StringUtils.repeat(" ", (int)Math.floor(ncols - value.length())/2) +
                        value;
                prov += value + StringUtils.repeat(" ", ncols - prov.length());
                return prov;
            }
            if (horizontalAlignment==HORIZONTAL_ALIGNMENT_RIGHT){
                return StringUtils.repeat(" ", ncols - value.length()) + value;
            }
            throw new Exception("Valor inválido para parámetro : horizontalAlignment");

        }
    }

    public String fillNColsFieldWithValue(String value, int ncols) throws Exception{
        return fillNColsFieldWithValue(value, ncols, HORIZONTAL_ALIGNMENT_LEFT);
    }

}