package contabilidad.entity.validation.asiento;

import contabilidad.entity.Apunte;
import contabilidad.entity.Asiento;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.text.SimpleDateFormat;

public class AsientoBeanValidator implements ConstraintValidator<AsientoValidation, Asiento> {

    @Override
    public boolean isValid(Asiento asiento, ConstraintValidatorContext context) {
        boolean isvalid = true;
        if (asiento == null) {
            return false;
        }

        boolean isEjercicioBloqueado = false;
        if (asiento.getEjercicio().getBloqueado()==null){
            isEjercicioBloqueado = false;
        }else{
            isEjercicioBloqueado = asiento.getEjercicio().getBloqueado();
        }
        if (isEjercicioBloqueado){
            context.buildConstraintViolationWithTemplate("No se puede modificar un asiento perteneciente a un ejercicio bloqueado").addConstraintViolation();
            isvalid = false;
            return isvalid;
        }
        Integer annoEjercicio = asiento.getEjercicio().getAnno();
        Integer annoAsiento = Integer.valueOf(new SimpleDateFormat("yyyy").format(asiento.getFecha()));
;        if (annoAsiento.compareTo(annoEjercicio)!=0){
            context.buildConstraintViolationWithTemplate("El asiento corresponde a un ejercicio diferente del activo. Corregir fecha o seleccionar nuevo ejercicio activo.").addConstraintViolation();
            isvalid = false;
            return isvalid;
        }

        //la suma de los apuntes debe y haber debe ser igual
        double sumaDebe = 0, sumaHaber = 0;
        for (int i = 0; i < asiento.getApuntes().size(); i++) {
            Apunte a = asiento.getApuntes().get(i);
            if (!a.getDebeHaber()){
                sumaHaber += a.getImporte();
            }
            if (a.getDebeHaber()){
                sumaDebe += a.getImporte();
            }
        }
        if (Math.abs(sumaDebe - sumaHaber)>0.001){
            context.buildConstraintViolationWithTemplate("La suma de debe y haber debe coincidir").addConstraintViolation();
            isvalid = false;
        }

        if (asiento.getEjercicio()==null){
            context.buildConstraintViolationWithTemplate("Indicar ejercicio").addConstraintViolation();
            isvalid = false;
        }

        if (asiento.getNumAsiento()==null){
            context.buildConstraintViolationWithTemplate("Nº de Asiento no puede estar vacío").addConstraintViolation();
            isvalid = false;
        }

        if (asiento.getFecha()==null){
            context.buildConstraintViolationWithTemplate("Indicar fecha de asiento").addConstraintViolation();
            isvalid = false;
        }


        return isvalid;
    }
}