package contabilidad.jmx.dimoni;

/*
Las tablas desde donde se ha de importar la informacion son:
CAPTE
CSUBC
CRIVA
CRET0
FTERC?
CCONC

 */

import contabilidad.beans.AsientoBean;
import contabilidad.beans.SaldosBean;
import contabilidad.entity.*;
import io.jmix.core.DataManager;
import io.jmix.core.FetchPlan;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jmx.export.annotation.*;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.ResultSet;
import java.util.*;

@Component("contabi_DimoniImportacionManagementFacade")
@ManagedResource(description = "JMX bean for some settings", objectName = "jmix.data:type=DimoniImportacionBean")
public class DimoniImportacionManagementFacade {

    @Autowired
    private DataManager dataManager;
    @Autowired
    private AsientoBean asientoBean;
    @Autowired
    private SaldosBean saldosBean;

    private String getNombreDesdeNifParaEmpresa(String codigoEmpresaDimoni, String nif) throws Exception{
        String nsql = "select * from fterc where Codigo = '"+nif+"'";
        ResultSet r = DimoniConnector.getResultSet(nsql);
        ArrayList<String> names = new ArrayList();
        String domicili1 = null;
        String codPostal = null;
        String comunidad = null;
        String municipio = null;
        String provincia = null;
        String telefono1 = null;
        String email = null;
        while(r.next()){
            names.add(r.getString("Nombre"));
            domicili1 = r.getString("Siglas") + " " + r.getString("Domicili") + " " + r.getString("Numero");
            codPostal = r.getString("CodPosta");
            comunidad = r.getString("Comunida");
            municipio = r.getString("Localida");
            provincia = r.getString("Provinci");
            telefono1 = r.getString("Telefon1");
            email = r.getString("Email");
        }
        r.close();
        if (domicili1==null) domicili1 = "";
        if (codPostal==null) codPostal = "";
        if (comunidad==null) comunidad = "";
        if (municipio==null) municipio = "";
        if (provincia==null) provincia = "";
        if (telefono1==null) telefono1 = "";
        if (email==null) email = "";

        String nombre = null;
        if (names.size()==1){
            nombre =  names.get(0);
        }
        else if (names.size()>1){
//            throw new Exception("Error: mas de un nombre encontrado para :" + codigoEmpresaDimoni + " y codigo:" + nif);
            nombre = names.get(0);
        }else{
            nombre = "NOMBRE NO ENCONTRADO EN FTERC";
        }
        //gestion de Titulares
        try{
            String eql = "select t from contabi_Titular t where t.nif = :nif";
            Titular t = dataManager.load(Titular.class).query(eql).parameter("nif", nif).one();
        }catch(Exception exc){
            //creamos nuevo titular
            Titular t = dataManager.create(Titular.class);
            t.setNombre(nombre);
            t.setNif(nif);
            t.setDireccion1(domicili1);
            t.setCodigoPostal(codPostal);
            t.setComunidad(comunidad);
            t.setEmail(email);
            t.setMunicipio(municipio);
            t.setProvincia(provincia);
            t.setTelefono(telefono1);
            dataManager.save(t);
        }
        return nombre;
    }

    private Empresa getEmpresaDesdeString(String s) throws Exception{
        String eql = "select e from contabi_Empresa e where e.nombre like :emp";
        List<Empresa> l = dataManager.load(Empresa.class).query(eql).parameter("emp","%"+s+"%").list();
        if (l.size()==1){
            return l.get(0);
        }else{
            throw new Exception("Mas de una empresa para cadena " + s);
        }
    }

    private NombreCuentaContable getNCCDesdeString(String codigo, String nombre, Hashtable<String, List<NombreCuentaContable>> nombres) throws Exception{
        List<NombreCuentaContable> l = nombres.get(codigo);
        for (int i = 0; i < l.size(); i++) {
            NombreCuentaContable ncc = l.get(i);
            if (ncc.getNombre().compareTo(nombre)==0){
                return ncc;
            }
        }
        return null;
    }

//    @ManagedOperation(description = "Importador de saldos de cuentas y ejercicios")
//    @ManagedOperationParameters({
//            @ManagedOperationParameter(name = "annoInt", description = "Anno de importacion"),
//            @ManagedOperationParameter(name = "empresa", description = "prefijo de empresa")})
//    public void realizaImportacionDeSaldosParaCuentasYEjercicios(Integer annoInt, String prefijoEmpresa) throws Exception{
//        Empresa emp = getEmpresaDesdeString(prefijoEmpresa);
//        Ejercicio ej = devuelveInstanciaDeEjercicioParaEmpresaYAnno(annoInt, emp);
//        List<CuentaContable> cc = dataManager.load(CuentaContable.class)
//                .query("select cc from contabi_CuentaContable cc join cc.planContable pc where pc.id = :pcid " +
//                        "order by cc.codigo")
//                .parameter("pcid", ej.getPlanContable().getId())
//                .list();
//
//        Connection conn = DimoniConnector.getConnection(true);
//        String grpId = devuelveGrpIdDeEmpresaEjercicio(emp, ej);
//        for (int i = 0; i < cc.size(); i++) {
//            SaldoCuentaEjercicio sce = dataManager.create(SaldoCuentaEjercicio.class);
//            CuentaContable c = cc.get(i);
//            sce.setCuentaContable(c);
//            sce.setEjercicio(ej);
//
//
//            String sql = "select * from CSUBC where Codigo = '" + c.getCodigo() + "' and GRP_ID = '" + grpId + "' " +
//                    "";
//
//            ResultSet r = DimoniConnector.getResultSet(sql, conn);
//            while(r.next()){
//                sce.setSaldoInicial(r.getDouble("SdoInDeb"));
//
//            }
//            r.close();
//            dataManager.save(sce);
//
//        }
//        conn.close();
//
//
//
//    }

    @ManagedOperation(description = "Importador de saldos de cuentas y ejercicios")
    @ManagedOperationParameters({
            @ManagedOperationParameter(name = "annoFrom", description = "Anno de importacion"),
            @ManagedOperationParameter(name = "annoTo", description = "Anno de importacion"),
            @ManagedOperationParameter(name = "empresa", description = "prefijo de empresa")})
    public void realizaImportacionDeSaldosParaCuentasYEjercicios(Integer annoFrom, Integer annoTo, String prefijoEmpresa) throws Exception{
        for (int i = annoFrom; i < (annoTo+1); i++) {
            realizaImportacionDeSaldosParaCuentasYEjercicios(i, prefijoEmpresa);
        }
    }


    public void realizaImportacionDeSaldosParaCuentasYEjercicios(Integer annoInt, String prefijoEmpresa) throws Exception{
        Empresa emp = getEmpresaDesdeString(prefijoEmpresa);
        Ejercicio ej = devuelveInstanciaDeEjercicioParaEmpresaYAnno(annoInt, emp);

        //borrando antes todos los saldos que previamente pudieran estar registrados
        saldosBean.realizaEliminacionDeSaldosCuentaEjercicioParaEjercicio(ej);


        List<CuentaContable> cc = dataManager.load(CuentaContable.class)
                .query("select cc from contabi_CuentaContable cc join cc.planContable pc where pc.id = :pcid " +
                        "order by cc.codigo")
                .parameter("pcid", ej.getPlanContable().getId())
                .list();
        Hashtable<String, CuentaContable> cuentasHt = new Hashtable<String, CuentaContable>();
        String codigos = "(";
        for (int i = 0; i < cc.size(); i++) {
            if (i>0) codigos += ", ";
            codigos += "'" + cc.get(i).getCodigo() + "'";
            cuentasHt.put(cc.get(i).getCodigo(), cc.get(i));
        }
        codigos += ")";

        String grpId = devuelveGrpIdDeEmpresaEjercicio(emp, ej);
        String sql = "select * from CSUBC where Codigo in " + codigos + " and GRP_ID = '" + grpId + "' " +
                "";

        Connection conn = DimoniConnector.getConnection(true);
        ResultSet r = DimoniConnector.getResultSet(sql, conn);
        List<SaldoCuentaEjercicio> saveable = new ArrayList<>();
        while(r.next()){
            String codigo = r.getString("Codigo");
            CuentaContable c = cuentasHt.get(codigo);
            SaldoCuentaEjercicio sce = dataManager.create(SaldoCuentaEjercicio.class);
            sce.setCuentaContable(c);
            sce.setEjercicio(ej);
            sce.setSaldoInicialDeudor(r.getDouble("SdoInHab"));
            sce.setSaldoInicialAcreedor(r.getDouble("SdoInDeb"));
            dataManager.save(sce);
        }
        r.close();
        conn.close();





    }

    @ManagedOperation(description = "Importador de datos de la version de Excact Dimoni")
    public void realizaImportacionPlanesContables_Ejercicios_Cuentas_Y_ConceptosApuntes() throws Exception{

        //creamos las dos empresas
        Empresa emp1 = dataManager.create(Empresa.class);
        emp1.setNombre("CIA.GRAL.CONSTRUCCIONES GUADALEST, SL");
        emp1.setCif("B08377079");
        emp1.setDireccion1("CL ROSSELLO 216 13A");
        emp1.setDireccion2("08008 BARCELONA");
        emp1.setFechaAlta(new Date());
        emp1.setRazonSocial("GUADALEST");
        dataManager.save(emp1);
        emp1 = dataManager.create(Empresa.class);
        emp1.setNombre("MARVALLOS, SL");
        emp1.setCif("B65687766");
        emp1.setDireccion1("CL ROSSELLO 216 13A");
        emp1.setDireccion2("08008 BARCELONA");
        emp1.setFechaAlta(new Date());
        emp1.setRazonSocial("MARVALLOS");
        dataManager.save(emp1);

        String currGrpId = null;
        String currAnno = null;
        String currCta = null;
        String currDescrip = null;
        try{
            String nativeSql = "select * from csubc " +
                    " where GRP_ID like '%guadal%' or GRP_ID like '%marval%'" +
                    " order by GRP_ID, Codigo, Descrip ";
            Connection conn = DimoniConnector.getConnection(true);
            Hashtable<String, PlanContable> empresasPlanes = new Hashtable<String, PlanContable>();
            Hashtable<String, Ejercicio> empresasAnnos = new Hashtable<String, Ejercicio>();
            Hashtable<String, Empresa> empresas = new Hashtable<String,Empresa>();
            Hashtable<String, CuentaContable> empresasCuentas = new Hashtable<String, CuentaContable>();
            Hashtable<String, List<NombreCuentaContable>> nombres = new Hashtable<String, List<NombreCuentaContable>>();
            ResultSet rs = DimoniConnector.getResultSet(nativeSql, conn);
            while(rs.next()){
                String grp_id = rs.getString("GRP_ID");
                String anno = grp_id.substring(grp_id.length()-2);
                Integer annoInt = 2000 + new Integer(anno).intValue();
                String codigo = rs.getString("Codigo");
                String descrip = rs.getString("Descrip");
                if ((grp_id.indexOf("GUADAL")!=-1)||(grp_id.indexOf("MARVAL")!=-1)){
                    String empresa = grp_id.substring(2, grp_id.length()-3);
                    Empresa emp;
                    if (!empresas.containsKey(empresa)){
                        Empresa e = getEmpresaDesdeString(empresa);
                        empresas.put(empresa, e);
                    }
                    emp = empresas.get(empresa);
                    PlanContable pc;
                    if (!empresasPlanes.containsKey(empresa)){
                        pc = dataManager.create(PlanContable.class);
                        pc.setDescripcion("PLAN CONTABLE " + empresa);
                        pc.setNombre(empresa);
                        pc.setProfundidad(8);
                        pc.setTipo("PLAN_CONTABLE_ESTANDAR");

                        empresasPlanes.put(empresa, pc);
                    }
                    pc = empresasPlanes.get(empresa);
                    String empresaAnno = empresa + new Integer(annoInt).toString();
                    Ejercicio ej;
                    if (!empresasAnnos.containsKey(empresaAnno)){
                        ej = dataManager.create(Ejercicio.class);
                        ej.setEmpresa(emp);
                        ej.setAnno(annoInt);
                        ej.setPlanContable(pc);
                        //ej.setFechaInicio(new Date(annoInt, 0, 1));
                        ej.setFechaInicio(DateUtils.parseDate("01-01-" + annoInt, "dd-MM-yyyy"));
                        //ej.setFechaFin(new Date(annoInt, 11, 31));
                        ej.setFechaFin(DateUtils.parseDate("31-12-" + annoInt, "dd-MM-yyyy"));
                        empresasAnnos.put(empresaAnno, ej);
                    }
                    ej = empresasAnnos.get(empresaAnno);

                    CuentaContable cc;
                    if (!empresasCuentas.containsKey(empresa+"_"+codigo)){
                        cc = dataManager.create(CuentaContable.class);
                        cc.setCodigo(codigo);
                        cc.setEmpresa(emp);
                        cc.setDescripcion(descrip);
                        cc.setNombre(descrip);
                        cc.setPlanContable(pc);
                        NombreCuentaContable ncc = dataManager.create(NombreCuentaContable.class);
                        ncc.setCuentaContable(cc);
                        ncc.setNombre(descrip);
                        ncc.setFechaDesde(ej.getFechaInicio());

                        ArrayList al = new ArrayList();
                        al.add(ncc);
                        nombres.put(codigo, al);
                        empresasCuentas.put(empresa+"_"+codigo, cc);
                    }else{
                        cc  = empresasCuentas.get(empresa+"_"+codigo);
                        NombreCuentaContable ncc = getNCCDesdeString(codigo, descrip, nombres);
                        if (ncc!=null){
                            if (ncc.getFechaHasta()==null){
                                //continuamos con el mismo
                            }else{
                                //se está reestableciendo el nombre, pero no podemos tomar el antiguo por tanto
                                //creamos uno nuevo
                                NombreCuentaContable n_ncc = dataManager.create(NombreCuentaContable.class);
                                n_ncc.setCuentaContable(cc);
                                n_ncc.setNombre(descrip);
                                n_ncc.setFechaDesde(ej.getFechaInicio());
                                List<NombreCuentaContable> l = nombres.get(codigo);
                                NombreCuentaContable ncc_old = l.get(l.size()-1);
                                ncc_old.setFechaHasta(DateUtils.addDays(ej.getFechaInicio(), -1));
                                l.add(n_ncc);
                                cc.setNombre(n_ncc.getNombre());
                            }
                        }else{
                            //generamos uno nuevo
                            NombreCuentaContable n_ncc = dataManager.create(NombreCuentaContable.class);
                            n_ncc.setCuentaContable(cc);
                            n_ncc.setNombre(descrip);
                            n_ncc.setFechaDesde(ej.getFechaInicio());
                            List<NombreCuentaContable> l = nombres.get(codigo);
                            NombreCuentaContable ncc_old = l.get(l.size()-1);
                            ncc_old.setFechaHasta(DateUtils.addDays(ej.getFechaInicio(), -1));
                            l.add(n_ncc);
                            cc.setNombre(n_ncc.getNombre());
                        }


                    }




                }
            }


            rs.close();
            conn.close();

            //guardando datos
            Iterator<String> planes = empresasPlanes.keySet().iterator();
            while(planes.hasNext()){
                PlanContable pc = empresasPlanes.get(planes.next());
                dataManager.save(pc);
            }


            //importamos conceptos de apuntes
            String sql = "select * from CCONC where GRP_ID = 'E_GUADALJ' or GRP_ID = 'E_MARVALJ' order by GRP_ID, Codigo";
            conn = DimoniConnector.getConnection(true);
            rs = DimoniConnector.getResultSet(sql, conn);
            PlanContable pcGuadal = dataManager.load(PlanContable.class).query("select pc from contabi_PlanContable pc where pc.nombre = 'GUADAL'").one();
            PlanContable pcMarval = dataManager.load(PlanContable.class).query("select pc from contabi_PlanContable pc where pc.nombre = 'MARVAL'").one();
            Hashtable<String, PlanContable> planesContables = new Hashtable();
            planesContables.put("E_GUADALJ",pcGuadal);
            planesContables.put("E_MARVALJ", pcMarval);
            while(rs.next()){
                String grpId = rs.getString("GRP_ID");
                String codigo = rs.getString("Codigo");
                String descrip = rs.getString("Descrip");
                ConceptoApunte ca = dataManager.create(ConceptoApunte.class);
                ca.setPlanContable(planesContables.get(grpId));
                ca.setCodigo(codigo);
                ca.setDescripcion(descrip);
                dataManager.save(ca);

            }

            //seguimos guardando datos
            Iterator<String> ejercicios = empresasAnnos.keySet().iterator();
            while(ejercicios.hasNext()){
                Ejercicio ej = empresasAnnos.get(ejercicios.next());
                dataManager.save(ej);
            }
            Iterator<String> cuentas = empresasCuentas.keySet().iterator();
            while(cuentas.hasNext()){
                CuentaContable cc = empresasCuentas.get(cuentas.next());
                dataManager.save(cc);
            }
            Iterator<String> nn = nombres.keySet().iterator();
            while(nn.hasNext()){
                List<NombreCuentaContable> l = nombres.get(nn.next());
                for (int i = 0; i < l.size(); i++) {
                    NombreCuentaContable cc = l.get(i);
                    dataManager.save(cc);
                }
            }
        }catch(Exception exc){
            throw exc;
        }


    }

    private String devuelveGrpIdDeEmpresaEjercicio(Empresa e, Ejercicio ej) throws Exception{
        String idMarvallos = devuelveIdDeEmpresaParaNombre("MARVAL");
        String idGuadal = devuelveIdDeEmpresaParaNombre("GUADAL");
        String[] uids = new String[]{idMarvallos, idGuadal} ;
        String[] grpids = new String[]{"MARVAL", "GUADAL"};
        String grpId = "E_" + grpids[Arrays.asList(uids).indexOf(e.getId().toString().replace("-",""))] + "J" + ej.getAnno().toString().substring(2);
        return grpId;
    }

    private String devuelveIdDeEmpresaParaNombre(String nombre) throws Exception{
        List<Empresa> emps = dataManager.load(Empresa.class)
                .query("select e from contabi_Empresa e where e.nombre like :n")
                .parameter("n", "%" + nombre + "%")
                .list();
        if (emps.size()==1){
            return emps.get(0).getId().toString().replace("-","");
        }else{
            throw new Exception("Error al seleccionar empresa desde nombre " + nombre);
        }
    }

    private String devuelveGrpIdDeEmpresaSinEjercicio(Empresa e) throws Exception{
        String idMarvallos = devuelveIdDeEmpresaParaNombre("MARVAL");
        String idGuadal = devuelveIdDeEmpresaParaNombre("GUADAL");
        String[] uids = new String[]{idMarvallos, idGuadal} ;
        String[] grpids = new String[]{"MARVAL", "GUADAL"};
        String grpId = "E_" + grpids[Arrays.asList(uids).indexOf(e.getId().toString().replace("-",""))] + "J";
        return grpId;
    }

    private CuentaContable getCuentaContableDesdeCodigoYEjercicio(String codigo, Ejercicio ej) throws Exception{
        String eql = "select cc from contabi_CuentaContable cc join cc.planContable pc where pc.id = :pcid and " +
                "cc.codigo like :codigo ";

        List<CuentaContable> cc = dataManager.load(CuentaContable.class).query(eql)
                .parameter("pcid", ej.getPlanContable().getId())
                .parameter("codigo", codigo)
                .list();
        if (cc.size()==1){
            return cc.get(0);
        }else{
            throw new Exception("Error al seleccionar cuenta para datos:" + codigo + ", Ejercicio:" + ej.getDescriptivo());
        }
    }

    private Ejercicio devuelveInstanciaDeEjercicioParaEmpresaYAnno(Integer annoInt, Empresa emp) throws Exception{
        List<Ejercicio> ejercicios = dataManager.load(Ejercicio.class)
                .query("select ej from contabi_Ejercicio ej join ej.empresa emp where emp.id = :empid and ej.anno = :anno order by ej.anno")
                .parameter("empid", emp.getId()).fetchPlan(fpb->fpb.addFetchPlan(FetchPlan.BASE).add("planContable"))
                .parameter("anno", annoInt)
                .list();
        if (ejercicios.size()==1) {
            try{
                Ejercicio ej = ejercicios.get(0);
                return ej;
            }catch(Exception exc){
                throw new Exception("Ejercicio para datos no hallado : " + annoInt + " - " + emp.getNombre());
            }
        }else{
            throw new Exception("Ejercicio para datos no hallado : " + annoInt + " - " + emp.getNombre());
        }
    }


    /*
    SQL SEGUIMIENTO ASIENTOS EJERCICIOS DE EMPRESAS
    SELECT max(a.fecha) FROM contabi2.contabi_asiento a
inner join contabi2.contabi_ejercicio ej on a.ejercicio_id = ej.id
inner join contabi2.contabi_plan_contable pc on ej.plan_contable_id = pc.id
where pc.nombre like 'marval';
     */

    @ManagedOperation(description = "Importador de asientos de la version de Exact Dimoni")
    @ManagedOperationParameters({
            @ManagedOperationParameter(name = "annoFrom", description = "Anno de importacion"),
            @ManagedOperationParameter(name = "annoTo", description = "Anno de importacion"),
            @ManagedOperationParameter(name = "empresa", description = "prefijo de empresa")})
    public void realizaImportacionesAsientosDeEjercicios(Integer annoFrom, Integer annoTo, String empresa) throws Exception{
        for (int i = annoFrom; i < (annoTo+1); i++) {
            int anno = i;
            realizaImportacionAsientosDeEjercicios(anno, empresa);
        }
    }


    public void realizaImportacionAsientosDeEjercicios(Integer annoInt, String empresa) throws Exception{
        List<Empresa> empresas = dataManager.load(Empresa.class)
                .query("select e from contabi_Empresa e where e.nombre like :emp")
                .parameter("emp", "%"+empresa+"%")
                .list();
        for (int i = 0; i < empresas.size(); i++) {
            Empresa emp = empresas.get(i);

            List<Ejercicio> ejercicios = dataManager.load(Ejercicio.class)
                    .query("select ej from contabi_Ejercicio ej join ej.empresa emp where emp.id = :empid and ej.anno = :anno order by ej.anno")
                    .parameter("empid", emp.getId()).fetchPlan(fpb->fpb.addFetchPlan(FetchPlan.BASE).add("planContable"))
                    .parameter("anno", annoInt)
                    .list();
            for (int j = 0; j < ejercicios.size(); j++) {
                Ejercicio ej = ejercicios.get(j);
                String grpId = devuelveGrpIdDeEmpresaEjercicio(emp, ej);
                try {
                    HashMap<String, Asiento> asientos = new HashMap<String, Asiento>();
                    HashMap<String, Apunte> apuntes = new HashMap<String, Apunte>();
                    HashMap<String, List<MovtoIva>> movtosIva = new HashMap<String, List<MovtoIva>>();
                    HashMap<String, List<MovtoIrpf>> movtosIrpf = new HashMap<String, List<MovtoIrpf>>();

                    HashMap<Asiento, String[]> numAstosCoordsIvaIrpf = new HashMap<Asiento, String[]>();
//                    HashMap<String, String[]> numAstosCoordsIrpf = new HashMap<String, String[]>();

                    String nativeSql = "select * from capte where GRP_ID = '" + grpId + "' order by NumAsto, NumApte";
                    Connection conn = DimoniConnector.getConnection(true);
                    ResultSet r = DimoniConnector.getResultSet(nativeSql);

                    HashMap<String, Asiento> astos = new HashMap<String, Asiento>();
                    String descripAcum = "";
                    while(r.next()){

                        String riserie = r.getString("RISerie");
                        String rinumer = r.getString("RINumer");
                        String risigno = r.getString("RISigno");

                        String numAsto = r.getString("NumAsto");

                        String numApte = r.getString("NumApte");

                        Asiento a;

                        if (!asientos.containsKey(numAsto)){
                            descripAcum = "";
                            a = dataManager.create(Asiento.class);
                            a.setFuente("DIMONI");
                            a.setEjercicio(ej);
                            a.setDescripcion("");
                            a.setFecha(r.getDate("FecApte"));
                            a.setFechaRegistro(new Date());
                            a.setNumAsiento(new Integer(numAsto));

                            asientos.put(numAsto, a);
                        }
                        a = asientos.get(numAsto);

                        if (!numAstosCoordsIvaIrpf.containsKey(a)){
                            numAstosCoordsIvaIrpf.put(a, new String[]{riserie, rinumer, risigno});
                        }

                        String desc  = r.getString("AmplCon");
                        if (a.getDescripcion().indexOf(desc)==-1){
                            if (descripAcum.length()==0){
                                descripAcum += desc;
                            }else{
                                descripAcum += " - " + desc;
                            }

                            a.setDescripcion(descripAcum);
                        }

                        if (a.getDescripcion() == null){

                            a.setDescripcion(desc);
                        }
                        astos.put(numAsto, a);
                        Apunte ap = dataManager.create(Apunte.class);
                        ap.setAsiento(a);

                        String concepto = r.getString("Concepto");
                        ConceptoApunte ca = dataManager.load(ConceptoApunte.class).query("select ca from contabi_ConceptoApunte ca where ca.codigo = :codigo").parameter("codigo", concepto).one();
                        ap.setConceptoApunte(ca);

                        ap.setNumApunte(new Integer(numApte));
                        if (r.getString("DebeHab").compareTo("D")==0){
                            ap.setDebeHaber(true);
                            //a.getApuntesDebe().add(ap);
                        }else{
                            ap.setDebeHaber(false);
                            //a.getApuntesHaber().add(ap);
                        }
                        CuentaContable cc = getCuentaContableDesdeCodigoYEjercicio(r.getString("Scta"), ej);
                        ap.setCuentaContable(cc);
                        ap.setImporte(r.getDouble("Importe"));
                        ap.setAmpliacion(r.getString("AmplCon"));
                        apuntes.put(numAsto+numApte,ap);


                    }
                    r.close();

                    //movtos iva irpf
                    Iterator<Asiento> asientosIterator = numAstosCoordsIvaIrpf.keySet().iterator();
                    while(asientosIterator.hasNext()){
                        Asiento a = asientosIterator.next();
                        String numAsto = a.getNumAsiento().toString();
                        String[] coords = numAstosCoordsIvaIrpf.get(a);
                        String riserie = coords[0];
                        String rinumer = coords[1];
                        String risigno = coords[2];
                        //movtos iva
                        //fecha f es la fecha de factura
                        //FecFra es la fecha de contabilizacion en 472..99
                        String sql_iva = "select distinct civa1.Nif, civa1.FecFra, civa1.FechaF, civa1.RefExt, civa1.TipoImp, civa1.CuotaIva, civa1.BaseImp, civa1.TipoFra  from criva as civa1\n" +
                                " where civa1.Serie = '" + riserie + "' \n" +
                                "\tand civa1.NumFra = '" + rinumer + "' \n" +
                                "\tand civa1.RISigno = '" + risigno + "' \n";
                        ResultSet riva = DimoniConnector.getResultSet(sql_iva);
                        while(riva.next()){
                            MovtoIva miva = dataManager.create(MovtoIva.class);
                            miva.setAsiento(a);
                            miva.setNif(riva.getString("Nif"));
                            String nombre = getNombreDesdeNifParaEmpresa(devuelveGrpIdDeEmpresaSinEjercicio(emp), miva.getNif());
                            miva.setNombre(nombre);

                            TipoMovtoIva tiva = TipoMovtoIva.fromId(riva.getString("TipoFra"));
                            miva.setTipoMovto(tiva);

                            miva.setFechaDeclIva(riva.getDate("FecFra"));
                            miva.setFechaDeFactura(riva.getDate("FechaF"));
                            miva.setNumFra(riva.getString("RefExt"));
                            miva.setTipoPorcentaje(riva.getDouble("TipoImp")/100);
                            miva.setCuota(riva.getDouble("CuotaIva"));
                            miva.setImporteBase(riva.getDouble("BaseImp"));
                            if (!movtosIva.containsKey(grpId+numAsto)){
                                movtosIva.put(grpId+numAsto, new ArrayList());
                            }
                            List l = movtosIva.get(grpId+numAsto);
                            l.add(miva);
                        }
                        riva.close();

                        //movimientos de irpf
                        String ej_aj = devuelveGrpIdDeEmpresaEjercicio(emp, ej);
                        ej_aj = ej_aj.substring(0,ej_aj.length()-2);
                        String sql_irpf;
                        sql_irpf = "select distinct cirpf.Nif,cirpf.Retencio,cirpf.FecApte,cirpf.BaseImp,cirpf.RefExt, cirpf.TipoRet, cirpf.Clave \n" +
                                " from cret0 as cirpf \n" +
                                " where cirpf.GRP_ID  = '" + ej_aj + "' \n" +
                                "\tand cirpf.RISigno = '"+risigno+"'\t\n" +
                                "\tand cirpf.Serie = '"+riserie+"' \n" +
                                "\tand cirpf.RINum = '"+rinumer+"'";

                        ResultSet rirpf = DimoniConnector.getResultSet(sql_irpf);
                        while(rirpf.next()){
                            MovtoIrpf mirpf = dataManager.create(MovtoIrpf.class);
                            mirpf.setAsiento(a);
                            mirpf.setNif(rirpf.getString("Nif"));
                            SubclaveIrpf sirpf = SubclaveIrpf.fromId(rirpf.getString("Clave"));
                            mirpf.setSubclave(sirpf);
                            String nombre = getNombreDesdeNifParaEmpresa(devuelveGrpIdDeEmpresaSinEjercicio(emp), mirpf.getNif());
                            mirpf.setNombre(nombre);

                            mirpf.setCuota(rirpf.getDouble("Retencio"));
                            mirpf.setFecha(rirpf.getDate("FecApte"));
                            mirpf.setFechaDeclIrpf(rirpf.getDate("FecApte"));
                            mirpf.setBase(rirpf.getDouble("BaseImp"));
                            mirpf.setNumFra(rirpf.getString("RefExt"));
                            mirpf.setTipoPorcentaje(rirpf.getDouble("TipoRet"));
                            if (!movtosIrpf.containsKey(grpId+numAsto)){
                                movtosIrpf.put(grpId+numAsto, new ArrayList());
                            }
                            List l = movtosIrpf.get(grpId+numAsto);
                            l.add(mirpf);
                        }
                        rirpf.close();
                    }

//                    Iterator<String> numastos = astos.keySet().iterator();
//                    int currasto = -1;
//                    while(numastos.hasNext()){
//                        currasto++;
//                        String numAsto = numastos.next();
//                        Asiento a = astos.get(numAsto);
//                        //movimientos de iva
//                        String sql_iva = "select distinct civa1.Nif, civa1.FecFra, civa1.RefExt, civa1.TipoImp, civa1.CuotaIva, civa1.BaseImp  from capte as cap\n" +
//                                "inner join criva as civa1 on civa1.GRP_ID = cap.GRP_ID \n" +
//                                "\tand civa1.Serie = cap.RISerie \n" +
//                                "\tand civa1.NumFra = cap.RINumer\n" +
//                                "\tand civa1.RISigno = cap.RISigno\n" +
//                                "where cap.GRP_ID  like '" + devuelveGrpIdDeEmpresaEjercicio(emp, ej) + "'" +
//                                " and cap.NumAsto = '" + numAsto + "'";
//                        ResultSet riva = DimoniConnector.getResultSet(sql_iva);
//                        while(riva.next()){
//                            MovtoIva miva = dataManager.create(MovtoIva.class);
//                            miva.setAsiento(a);
//                            miva.setNif(riva.getString("Nif"));
//                            String nombre = getNombreDesdeNifParaEmpresa(devuelveGrpIdDeEmpresaSinEjercicio(emp), miva.getNif());
//                            miva.setNombre(nombre);
//
//                            miva.setFecha(riva.getDate("FecFra"));
//                            miva.setNumFra(riva.getString("RefExt"));
//                            miva.setTipoPorcentaje(riva.getDouble("TipoImp")/100);
//                            miva.setCuota(riva.getDouble("CuotaIva"));
//                            miva.setImporteBase(riva.getDouble("BaseImp"));
//                            if (!movtosIva.containsKey(grpId+numAsto)){
//                                movtosIva.put(grpId+numAsto, new ArrayList());
//                            }
//                            List l = movtosIva.get(grpId+numAsto);
//                            l.add(miva);
//                        }
//                        riva.close();
//                        //movimientos de irpf
//                        String sql_irpf;
//                        if (DimoniConnector.dbms.compareTo("mssql")==0){
//                            sql_irpf = "select distinct cirpf.Nif,cirpf.Retencio,cirpf.FecApte,cirpf.BaseImp,cirpf.RefExt, cirpf.TipoRet from capte as cap\n" +
//                                    "inner join cret0 as cirpf on cirpf.GRP_ID = left(cap.GRP_ID, len(cap.GRP_ID)-2) \n" +
//                                    "\tand cirpf.RISigno = cap.RISigno\t\n" +
//                                    "\tand cirpf.Serie = cap.RISerie \n" +
//                                    "\tand cirpf.RINum = cap.RINumer \n" +
//                                    "where cap.GRP_ID  like '" + devuelveGrpIdDeEmpresaEjercicio(emp, ej) + "' " +
//                                    " and cap.NumAsto like '" + numAsto + "'";
//                        }else{
//                            sql_irpf = "select distinct cirpf.Nif,cirpf.Retencio,cirpf.FecApte,cirpf.BaseImp,cirpf.RefExt, cirpf.TipoRet from capte as cap\n" +
//                                    "inner join cret0 as cirpf on cirpf.GRP_ID = left(cap.GRP_ID, length(cap.GRP_ID)-2) \n" +
//                                    "\tand cirpf.RISigno = cap.RISigno\t\n" +
//                                    "\tand cirpf.Serie = cap.RISerie \n" +
//                                    "\tand cirpf.RINum = cap.RINumer \n" +
//                                    "where cap.GRP_ID  like '" + devuelveGrpIdDeEmpresaEjercicio(emp, ej) + "' " +
//                                    " and cap.NumAsto like '" + numAsto + "'";
//                        }
//
//
//                        ResultSet rirpf = DimoniConnector.getResultSet(sql_irpf);
//                        while(rirpf.next()){
//                            MovtoIrpf mirpf = dataManager.create(MovtoIrpf.class);
//                            mirpf.setAsiento(a);
//                            mirpf.setNif(rirpf.getString("Nif"));
//                            String nombre = getNombreDesdeNifParaEmpresa(devuelveGrpIdDeEmpresaSinEjercicio(emp), mirpf.getNif());
//                            mirpf.setNombre(nombre);
//                            mirpf.setCuota(rirpf.getDouble("Retencio"));
//                            mirpf.setFecha(rirpf.getDate("FecApte"));
//                            mirpf.setBase(rirpf.getDouble("BaseImp"));
//                            mirpf.setNumFra(rirpf.getString("RefExt"));
//                            mirpf.setTipoPorcentaje(rirpf.getDouble("TipoRet"));
//                            if (!movtosIrpf.containsKey(grpId+numAsto)){
//                                movtosIrpf.put(grpId+numAsto, new ArrayList());
//                            }
//                            List l = movtosIrpf.get(grpId+numAsto);
//                            l.add(mirpf);
//                        }
//                        rirpf.close();
//                    }

                    //guardando datos
                    //asientos
                    Iterator<String> aa = asientos.keySet().iterator();
                    while(aa.hasNext()){
                        Asiento pc = asientos.get(aa.next());
                        dataManager.save(pc);
                    }
                    //apuntes
                    Iterator<String> aps = apuntes.keySet().iterator();
                    while(aps.hasNext()){
                        Apunte ap = apuntes.get(aps.next());
                        dataManager.save(ap);
                    }
                    //movtos iva
                    Iterator<String> ivas = movtosIva.keySet().iterator();
                    while(ivas.hasNext()){
                        List<MovtoIva> l = movtosIva.get(ivas.next());
                        for (int k = 0; k < l.size(); k++) {
                            MovtoIva miva = l.get(k);
                            dataManager.save(miva);
                        }
                    }
                    //movtos irpf
                    Iterator<String> irpfs = movtosIrpf.keySet().iterator();
                    while(irpfs.hasNext()){
                        List<MovtoIrpf> l = movtosIrpf.get(irpfs.next());
                        for (int k = 0; k < l.size(); k++) {
                            MovtoIrpf mirpf = l.get(k);
                            dataManager.save(mirpf);
                        }
                    }
                }catch(Exception exc){
                    throw exc;
                }
            }
        }

    }

    @ManagedOperation
    @ManagedOperationParameters({
            @ManagedOperationParameter(name = "anno", description = "Anno de eliminacion"),
            @ManagedOperationParameter(name = "empresa", description = "prefijo de empresa")})
    public void realizaEliminacionDeAsientosParaEjercicio(Integer anno, String empresa) throws Exception{
        Empresa emp = getEmpresaDesdeString(empresa);
        Ejercicio ej = devuelveInstanciaDeEjercicioParaEmpresaYAnno(anno, emp);
        String eql = "select a from contabi_Asiento a join a.ejercicio ej where ej.id = :ejid";
        List<Asiento> aa = dataManager.load(Asiento.class).query(eql)
                .parameter("ejid", ej.getId()).list();
        for (int i = 0; i < aa.size(); i++) {
            Asiento a = aa.get(i);
            boolean res = asientoBean.realizaEliminacionAsiento(a);
            if (!res){
                throw new Exception("El asiento " + a.getNumAsiento() + " del ejercicio " + ej.getDescriptivo() + " no pudo ser eliminado");
            }
        }
    }

    @ManagedOperation
    @ManagedOperationParameters({
            @ManagedOperationParameter(name = "anno", description = "Anno de eliminacion"),
            @ManagedOperationParameter(name = "empresa", description = "prefijo de empresa")})
    public boolean realizaEliminacionDeSaldosCuentaEjercicioParaEjercicio(Integer anno, String empresa) throws Exception{
        Empresa emp = getEmpresaDesdeString(empresa);
        Ejercicio ej = devuelveInstanciaDeEjercicioParaEmpresaYAnno(anno, emp);
        return saldosBean.realizaEliminacionDeSaldosCuentaEjercicioParaEjercicio(ej);
    }

//    private void gestionaRegistroDeTitular(String nif, String nombre) throws Exception{
//        String eql = "select t from contabi_Titular t where t.nif = :nif";
//        try{
//            Titular t = dataManager.load(Titular.class)
//                    .query(eql).parameter("nif", nif).one();
//        }catch(Exception exc){
//            //lo creamos y
//
//        }
//    }
}