package contabilidad.screen.asiento;

import contabilidad.beans.AsientoBean;
import contabilidad.beans.Txarh;
import contabilidad.entity.Apunte;
import contabilidad.entity.Ejercicio;
import contabilidad.entity.User;
import contabilidad.screen.textareareport.TextAreaReportScreen;
import io.jmix.core.DataManager;
import io.jmix.core.Entity;
import io.jmix.core.Id;
import io.jmix.core.LoadContext;
import io.jmix.core.querycondition.Condition;
import io.jmix.core.querycondition.LogicalCondition;
import io.jmix.core.querycondition.PropertyCondition;
import io.jmix.core.querycondition.PropertyConditionUtils;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.reports.entity.Report;
import io.jmix.reports.runner.ReportRunner;
import io.jmix.ui.Dialogs;
import io.jmix.ui.Notifications;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.action.Action;
import io.jmix.ui.action.DialogAction;
import io.jmix.ui.action.list.RemoveAction;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.CheckBox;
import io.jmix.ui.component.HasValue;
import io.jmix.ui.component.Table;
import io.jmix.ui.download.Downloader;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.model.DataContext;
import io.jmix.ui.screen.*;
import contabilidad.entity.Asiento;

import org.springframework.beans.factory.annotation.Autowired;

import javax.inject.Named;

import java.util.*;


@UiController("contabi_Asiento.browse")
@UiDescriptor("asiento-browse.xml")
@LookupComponent("asientoesTable")
public class AsientoBrowse extends StandardLookup<Asiento> {
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private CheckBox restriccionEjercicioActivo;
    @Autowired
    private CollectionLoader<Asiento> asientoesDl;
    @Autowired
    private Button duplicarBtn;
    @Autowired
    private Table<Asiento> asientoesTable;
    @Autowired
    private Notifications notifications;
    @Autowired
    private ScreenBuilders screenBuilders;
    @Autowired
    private AsientoBean asientoBean;
    @Autowired
    private Button printBtn;
    @Autowired
    private Button btnPdf;
    @Autowired
    private Txarh txarh;
    @Autowired
    private DataContext dataContext;
    @Autowired
    private ReportRunner reportRunner;
    @Autowired
    private Downloader downloader;
    @Autowired
    private Button removeBtn;
    @Autowired
    private Dialogs dialogs;

    @Subscribe
    public void onAfterInit(AfterInitEvent event) {
        duplicarBtn.addClickListener(e->{
            try {
                Asiento a = asientoesTable.getSingleSelected();
                if (a==null){
                    notifications.create().withDescription("Seleccionar asiento").show();
                    return;
                }
                a = dataManager.load(Asiento.class).id(a.getId()).fetchPlan("asiento-print").one();
                Asiento a2 = asientoBean.duplicarAsiento(a);
                screenBuilders.editor(Asiento.class, this).editEntity(a2).withScreenId("contabi_Asiento.edit")
                        .withOpenMode(OpenMode.NEW_TAB).build().show();
            } catch (Exception ex) {
                notifications.create().withCaption("Error").withDescription(ex.getMessage()).show();
                return;
            }
        });

        printBtn.addClickListener(e->{
            Ejercicio ej = ((User)currentAuthentication.getUser()).getEjercicioActivo();
            List<Asiento> aa = new ArrayList(asientoesTable.getSelected());
            if (aa.size() == 0){
                notifications.create().withDescription("Seleccionar al menos un Asiento").show();
                return;
            }
            try {
                String nums = "";
                String ss = "";
                for (int i = 0; i < aa.size(); i++) {
                    boolean printHeader = true;
                    if (i>0) printHeader = false;
                    Asiento a = aa.get(i);
                    nums += a.getNumAsiento().toString() + " ";
                    a = dataManager.load(Asiento.class).id(a.getId()).fetchPlan("asiento-print").one();
                    aa.set(i, a);
                }
                String s = AsientoEdit.getTxaReportAsiento(aa, ej, txarh);

                TextAreaReportScreen tars = (TextAreaReportScreen) screenBuilders.screen(this)
                        .withScreenId("contabi_TextAreaReportScreen")
                        .withOpenMode(OpenMode.NEW_TAB)
                        .build();
                tars.setContent(s);
                tars.setTitle("Asientos " + nums.trim());
                tars.show();
            } catch (Exception ex) {
                notifications.create().withDescription(ex.getMessage()).show();
                return;
            }
        });

        btnPdf.addClickListener(e->{
            Ejercicio ej = ((User)currentAuthentication.getUser()).getEjercicioActivo();
            List<Asiento> aa = new ArrayList(asientoesTable.getSelected());
            if (aa.size() == 0){
                notifications.create().withDescription("Seleccionar al menos un Asiento").show();
                return;
            }else if(aa.size()>1){
                notifications.create().withDescription("Termporalmente mediante esta opcion sólo se " +
                        "puede imprimir un asiento a la vez").show();
                return;
            }else{
                Asiento a = aa.get(0);
                a = dataManager.load(Asiento.class).id(a.getId()).fetchPlan("asiento-print").one();
                Hashtable<String, Object> params = new Hashtable<>();
                params.put("fechaRealizacion", new Date());
                params.put("nombreEmpresa", a.getEjercicio().getEmpresa().getNombre());
                params.put("numAsiento", a.getNumAsiento().toString());
                params.put("ejercicioAnno", a.getEjercicio().getAnno().toString());
                params.put("fechaAsiento", a.getFecha());
                params.put("descripcion", a.getDescripcion());
                List<Apunte> apuntes = a.getApuntes();
                Collections.sort(apuntes, new Comparator<Apunte>(){

                            @Override
                            public int compare(Apunte o1, Apunte o2) {
                                return o1.getNumApunte().compareTo(o2.getNumApunte());
                            }
                        }
                );
                params.put("apuntes", a.getApuntes());

                Report r = dataManager.load(Report.class).query("select r from report_Report r where r.name like :rn")
                        .parameter("rn", "Asiento Simple").one();
                byte[] bb = reportRunner.byReportEntity(r).withParams(params).run().getContent();
                downloader.download(bb, "Vista de Asiento " + a.getNumAsiento().toString() + "/" + a.getEjercicio().getAnno().toString() + ".pdf");
            }
        });

        removeBtn.addClickListener(e->{
            Asiento a = asientoesTable.getSingleSelected();
            if (a==null){
                notifications.create().withDescription("Seleccionar el asiento a eliminar").show();
                return;
            }
            dialogs.createOptionDialog().withCaption("Borrado de Asiento").withMessage("¿Está seguro que " +
                    "desea borrar el asiento seleccionado?")
                    .withActions(
                            new DialogAction(DialogAction.Type.YES).withHandler(ev->{
                                if (a.getEjercicio().getBloqueado() == null){
                                    a.getEjercicio().setBloqueado(false);
                                }
                                if (a.getEjercicio().getBloqueado()){
                                    notifications.create().withCaption("Ejercicio bloqueado")
                                            .withDescription("No se pueden borrar asientos asociados a un ejercicio bloqueado")
                                            .show();
                                    return;
                                }
                                try{
                                    asientoBean.realizaEliminacionAsiento(a);
                                    asientoesDl.load();
                                }catch(Exception exc){
                                    notifications.create().withCaption("Error al eliminar el asiento:" + exc.getMessage()).show();
                                    return;
                                }

                            }),
                            new DialogAction(DialogAction.Type.NO).withHandler(ev->{
                                //no hacemos nada
                            })

                    ).show();

        });
    }




    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        restriccionEjercicioActivo.setValue(true);
    }





    @Install(to = "asientoesDl", target = Target.DATA_LOADER)
    private List<Asiento> asientoesDlLoadDelegate(LoadContext<Asiento> loadContext) {
        List<Asiento> res = null;
        Ejercicio ej = ((User) currentAuthentication.getUser()).getEjercicioActivo();
        if (restriccionEjercicioActivo.isChecked()) {

            Integer anno = ((User) currentAuthentication.getUser()).getEjercicioActivo().getAnno();
            UUID id = ((User) currentAuthentication.getUser()).getEjercicioActivo().getId();

            PropertyCondition ejc = new PropertyCondition();
            ejc.setProperty("ejercicio.id");
            ejc.setOperation(PropertyCondition.Operation.EQUAL);
            ejc.setParameterName("id");
            ejc.setParameterValue(ej.getId());
            Condition c = loadContext.getQuery().getCondition();
            LogicalCondition lc = new LogicalCondition(LogicalCondition.Type.AND);
            lc.getConditions().add(c);
            lc.getConditions().add(ejc);
            loadContext.getQuery().setCondition(lc);

        }else{
            //restriccion de empresa
            PropertyCondition ejc = new PropertyCondition();
            ejc.setProperty("ejercicio.empresa.id");
            ejc.setOperation(PropertyCondition.Operation.EQUAL);
            ejc.setParameterName("id");
            ejc.setParameterValue(ej.getEmpresa().getId());
            Condition c = loadContext.getQuery().getCondition();
            LogicalCondition lc = new LogicalCondition(LogicalCondition.Type.AND);
            lc.getConditions().add(c);
            lc.getConditions().add(ejc);
            loadContext.getQuery().setCondition(lc);

        }
        res = dataManager.loadList(loadContext);
        return res;
    }

    @Subscribe("restriccionEjercicioActivo")
    public void onRestriccionEjercicioActivoValueChange(HasValue.ValueChangeEvent<Boolean> event) {
        asientoesDl.load();
    }





    
}