package contabilidad.screen.asiento;

import contabilidad.beans.AsientoBean;
import contabilidad.beans.Txarh;
import contabilidad.entity.*;
import io.jmix.core.DataManager;
import io.jmix.core.EntityStates;
import io.jmix.core.LoadContext;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Dialogs;
import io.jmix.ui.Notifications;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.UiComponents;
import io.jmix.ui.action.DialogAction;
import io.jmix.ui.component.*;
import io.jmix.ui.component.data.ValueSource;
import io.jmix.ui.component.data.value.ContainerValueSource;
import io.jmix.ui.component.impl.ComboBoxImpl;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.model.DataContext;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.*;

import io.jmix.ui.widget.JmixGrid;
import org.apache.commons.lang3.StringUtils;
import org.docx4j.wml.Numbering;
import org.springframework.beans.factory.annotation.Autowired;
import io.jmix.ui.component.ComboBox;

import javax.annotation.Nullable;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@UiController("contabi_Asiento.edit")
@UiDescriptor("asiento-edit.xml")
@EditedEntityContainer("asientoDc")
@MultipleOpen
public class AsientoEdit extends StandardEditor<Asiento> {

    @Autowired
    private UiComponents uiComponents;
    @Autowired
    private DataGrid<Apunte> apuntesTableDebe;
    @Autowired
    private DataGrid<Apunte> apuntesTableHaber;
    @Autowired
    private DataGrid<MovtoIva> movtoIvasTable;
    @Autowired
    private DataGrid<MovtoIrpf> movtoIrpfsTable;
    @Autowired
    private DataContext dataContext;
    @Autowired
    private CollectionContainer<Apunte> apuntesHaberDc;
    @Autowired
    private InstanceContainer<Asiento> asientoDc;
    @Autowired
    private CollectionContainer<Apunte> apuntesDebeDc;
    @Autowired
    private CollectionContainer<MovtoIva> movtoIvasDc;
    @Autowired
    private CollectionContainer<MovtoIrpf> movtoIrpfsDc;
    @Autowired
    private EntityStates entityStates;



    @Autowired
    private DataManager dataManager;
    @Autowired
    private Notifications notifications;
    @Autowired
    private Dialogs dialogs;
    @Autowired
    private AsientoBean asientoBean;
    @Autowired
    private TextField<Integer> numAsientoField;
    @Autowired
    private CurrentAuthentication currentAuthentication;

    @Autowired
    private Button duplicarAsientoBtn;
    @Autowired
    private ScreenBuilders screenBuilders;
    @Autowired
    private Txarh txarh;

    NumberFormat percent = NumberFormat.getPercentInstance();
    @Autowired
    private Button btnNuevoTitular;
    @Autowired
    private Button btnNuevoTitular2;

    @Subscribe
    public void onAfterInit(AfterInitEvent event) {
        apuntesTableDebe.setEditorEnabled(true);
        apuntesTableHaber.setEditorEnabled(true);
        apuntesTableHaber.getColumn("cuentaContable").setEditable(true);
        movtoIrpfsTable.setEditorEnabled(true);
        movtoIvasTable.setEditorEnabled(true);

        duplicarAsientoBtn.addClickListener(e->{
            try {
                Asiento a = asientoBean.duplicarAsiento(this.getEditedEntity());
                screenBuilders.editor(Asiento.class, this).editEntity(a).withScreenId("contabi_Asiento.edit")
                        .withOpenMode(OpenMode.NEW_TAB).build().show();
            } catch (Exception ex) {
                notifications.create().withCaption("Error").withDescription(ex.getMessage()).show();
                return;
            }
        });

        btnNuevoTitular.addClickListener(e->{
            screenBuilders.editor(Titular.class, this).newEntity().withOpenMode(OpenMode.DIALOG)
                    .build().show();
        });
        btnNuevoTitular2.addClickListener(e->{
            screenBuilders.editor(Titular.class, this).newEntity().withOpenMode(OpenMode.DIALOG)
                    .build().show();
        });
    }

    @Subscribe
    public void onInit(InitEvent event) {
        DataGrid.Column col = apuntesTableDebe.addGeneratedColumn("cuentaContable", e->{
            Apunte ap = e.getItem();
            if (ap.getCuentaContable() == null) return "";
            return ap.getCuentaContable().getCodigo() + " " + ap.getCuentaContable().getNombre();
        });
        col.setEditFieldGenerator(context->{
            ComboBox<CuentaContable> lookup = uiComponents.create(ComboBox.NAME);
            lookup.setPopupWidth("500px");
            lookup.setPageLength(10);
            lookup.setOptionCaptionProvider(AsientoEdit::getCaption);
            lookup.setOptionsList(getCuentasContables());
            lookup.setFilterMode(HasFilterMode.FilterMode.CONTAINS);
            lookup.setValueSource((ValueSource<CuentaContable>) ((DataGrid.EditorFieldGenerationContext)context).getValueSourceProvider().getValueSource("cuentaContable"));
             return lookup;
        });
        col = apuntesTableHaber.addGeneratedColumn("cuentaContable", e->{
            Apunte ap = e.getItem();
            if (ap.getCuentaContable() == null) return "";
            return ap.getCuentaContable().getCodigo() + " " + ap.getCuentaContable().getNombre();
        });
        col.setEditFieldGenerator(context->{
            ComboBox<CuentaContable> lookup = uiComponents.create(ComboBox.NAME);
            lookup.setPopupWidth("500px");
            lookup.setPageLength(10);
            lookup.setOptionCaptionProvider(AsientoEdit::getCaption);
            lookup.setOptionsList(getCuentasContables());
            lookup.setFilterMode(HasFilterMode.FilterMode.CONTAINS);
            lookup.setValueSource((ValueSource<CuentaContable>) ((DataGrid.EditorFieldGenerationContext)context).getValueSourceProvider().getValueSource("cuentaContable"));
            return lookup;
        });

        col = movtoIvasTable.addGeneratedColumn("tipoPorcentaje", e->{

            MovtoIva miva = e.getItem();
            return miva.getTipoPorcentaje()*100;
        });

        col.setEditFieldGenerator(context->{
            ComboBox<Double> lookup = uiComponents.create(ComboBox.NAME);
            lookup.setPopupWidth("150px");
            lookup.setPageLength(7);
            lookup.setOptionCaptionProvider(AsientoEdit::getCaptionTipoPorcentajeMIVA);
            lookup.setOptionsList(Arrays.asList(new Double[]{0.00,0.04,0.07, 0.10, 0.16,0.21,0.33}));
            lookup.setFilterMode(HasFilterMode.FilterMode.CONTAINS);
            lookup.setValueSource((ValueSource<Double>) ((DataGrid.EditorFieldGenerationContext)context).getValueSourceProvider().getValueSource("tipoPorcentaje"));
            lookup.addValueChangeListener(e->{
                InstanceContainer instanceContainer = ((ContainerValueSource)((ComboBoxImpl)e.getSource()).getValueSource()).getContainer();
                MovtoIva miva = (MovtoIva)instanceContainer.getItem();
                double nuevaCuota = miva.getImporteBase()*e.getValue();
                ((MovtoIva)instanceContainer.getItem()).setCuota(nuevaCuota);
                int y = 2;
            });
            return lookup;
        });

        //movtoirpf
        col = movtoIrpfsTable.addGeneratedColumn("tipoPorcentaje", e->{

            MovtoIrpf mirpf = e.getItem();
            return mirpf.getTipoPorcentaje()*100;
        });

        col.setEditFieldGenerator(context->{
            ComboBox<Double> lookup = uiComponents.create(ComboBox.NAME);
            lookup.setPopupWidth("150px");
            lookup.setPageLength(7);
            lookup.setOptionCaptionProvider(AsientoEdit::getCaptionTipoPorcentajeMIVA);
            lookup.setOptionsList(Arrays.asList(new Double[]{0.00,0.01,0.02, 0.07, 0.15,0.21}));
            lookup.setFilterMode(HasFilterMode.FilterMode.CONTAINS);
            lookup.setValueSource((ValueSource<Double>) ((DataGrid.EditorFieldGenerationContext)context).getValueSourceProvider().getValueSource("tipoPorcentaje"));
            lookup.addValueChangeListener(e->{
                InstanceContainer instanceContainer = ((ContainerValueSource)((ComboBoxImpl)e.getSource()).getValueSource()).getContainer();
                MovtoIrpf mirpf = (MovtoIrpf)instanceContainer.getItem();
                double nuevaCuota = mirpf.getBase()*e.getValue();
                ((MovtoIrpf)instanceContainer.getItem()).setCuota(nuevaCuota);
                int y = 2;
            });
            return lookup;
        });

        col = movtoIvasTable.addGeneratedColumn("nif", e->{
            MovtoIva miva = e.getItem();
            try{
                Titular t = dataManager.load(Titular.class).query("select t from contabi_Titular t where t.nif = :nif")
                        .parameter("nif", miva.getNif()).one();
                return t.getNif() + " - " + t.getNombre();
            }catch(Exception exc){
                return miva.getNif() + " - Titular no hallado";
            }

        });
        col.setEditFieldGenerator(context->{
            ComboBox<String> lookup = uiComponents.create(ComboBox.NAME);
            lookup.setPopupWidth("500px");
            lookup.setPageLength(10);
            lookup.setOptionCaptionProvider(AsientoEdit::getCaptionTitular);
            lookup.setOptionsList(getTitulares());
            lookup.setFilterMode(HasFilterMode.FilterMode.CONTAINS);
            lookup.setValueSource((ValueSource<String>) ((DataGrid.EditorFieldGenerationContext)context).getValueSourceProvider().getValueSource("nif"));
            return lookup;
        });
    }







    @Install(to = "apuntesTableHaber.cuentaContable", subject = "editFieldGenerator")
    private Field<?> apuntesTableHaberCuentaContableCodigoEditFieldGenerator(DataGrid.EditorFieldGenerationContext<Apunte> editorFieldGenerationContext) {
        ComboBox<CuentaContable> lookup = uiComponents.create(ComboBox.NAME);
        lookup.setPopupWidth("500px");
        lookup.setPageLength(10);
        lookup.setOptionCaptionProvider(AsientoEdit::getCaption);
        lookup.setOptionsList(getCuentasContables());
        lookup.setValueSource((ValueSource<CuentaContable>) editorFieldGenerationContext.getValueSourceProvider().getValueSource("cuentaContable"));
        return lookup;
    }

    @Install(to = "apuntesTableHaber.cuentaContable", subject = "columnGenerator")
    private String apuntesTableHaberCuentaContableColumnGenerator(DataGrid.ColumnGeneratorEvent<Apunte> columnGeneratorEvent) {
        if (columnGeneratorEvent.getItem().getCuentaContable()!=null){
            return getCaption(columnGeneratorEvent.getItem().getCuentaContable());
        }else{
            return "";
        }
    }


    
    public static String getTxaReportAsiento(List<Asiento> aa, Ejercicio ejActivo, Txarh txarh) throws Exception{
        String s = "";
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm");
        SimpleDateFormat sdf2 = new SimpleDateFormat("dd/MM/yyyy");
        DecimalFormat df = new DecimalFormat("#,##0.00");
        Collections.sort(aa, new Comparator<Asiento>() {
            @Override
            public int compare(Asiento o1, Asiento o2) {
                return o1.getNumAsiento().compareTo(o2.getNumAsiento());
            }
        });
        for (int i = 0; i < aa.size(); i++) {
            Asiento a = aa.get(i);
            boolean printHeader = true;
            if (i > 0) printHeader = false;
            if (printHeader) {
                Asiento adesde = aa.get(0);
                Asiento ahasta = aa.get(aa.size()-1);
                //cabeza fecha y hora
                s += txarh.fillNColsFieldWithValue(sdf.format(new Date()), 40) + txarh.breakLine();
                //nombre empresa
                s += txarh.fillNColsFieldWithValue(a.getEjercicio().getEmpresa().getNombre(), 50) + txarh.fillNColsFieldWithValue("(Ejercicio  " + ejActivo.getAnno().toString() + ")", 30, txarh.HORIZONTAL_ALIGNMENT_RIGHT) + txarh.breakLine();
                s += txarh.fillNColsFieldWithValue("Asientos - Comprobación", 30) + txarh.breakLine();
                s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-", 100), 100) + txarh.breakLine();
                s += txarh.fillNColsFieldWithValue("Asientos:", 40) + txarh.fillNColsFieldWithValue("Desde", 10) + txarh.fillNColsFieldWithValue("Hasta", 10) + txarh.breakLine();
                s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-", 100), 100) + txarh.breakLine();
                s += txarh.fillNColsFieldWithValue("Num.Asiento", 40) + txarh.fillNColsFieldWithValue(adesde.getNumAsiento().toString(), 15) + txarh.fillNColsFieldWithValue(ahasta.getNumAsiento().toString(), 15) + txarh.breakLine();
                s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-", 100), 100) + txarh.breakLine();
            }
            s += txarh.breakLine();

            int[] cols1 = new int[]{40, 20};
            int[] cols2 = new int[]{20, //asiento apunte
                    15, //codigo
                    40, //nombre cuenta
                    70,//ampl conc.
                    10, //importe
                    5 //debe haber
            };
            int addupcols1 = cols1[0] + cols1[1];
            int addupcols2 = cols2[0] + cols2[1] + cols2[2] + cols2[3] + cols2[4] + cols2[5];
            //encabezados
            s += txarh.fillNColsFieldWithValue("Fecha", cols1[0]) + txarh.breakLine();
            s += txarh.fillNColsFieldWithValue("Asto/Apte", cols2[0])
                    + txarh.fillNColsFieldWithValue("Codigo", cols2[1])
                    + txarh.fillNColsFieldWithValue("Nombre", cols2[2])
                    + txarh.fillNColsFieldWithValue("Concepto", cols2[3])
                    + txarh.fillNColsFieldWithValue("Importe", cols2[4])
                    + txarh.fillNColsFieldWithValue("D/H", cols2[5]) + txarh.breakLine();
            s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",addupcols2),addupcols2) + txarh.breakLine();
            //datos
            s += txarh.fillNColsFieldWithValue(sdf.format(a.getFecha()), cols1[0]) + txarh.breakLine();
            List<Apunte> apuntes = new ArrayList(a.getApuntes());
            Collections.sort(apuntes, txarh.comparatorAps);
            for (int j = 0; j < apuntes.size(); j++) {
                Apunte ap = apuntes.get(j);
                String dh = "D";
                if (ap.getDebeHaber()==false){
                    dh = "H";
                }
                s += txarh.fillNColsFieldWithValue(a.getNumAsiento().toString()+"/"+ap.getNumApunte().toString(), cols2[0])
                        + txarh.fillNColsFieldWithValue(ap.getCuentaContable().getCodigo(), cols2[1])
                        + txarh.fillNColsFieldWithValue(ap.getCuentaContable().getNombre(), cols2[2])
                        + txarh.fillNColsFieldWithValue(ap.getAmpliacion(), cols2[3])
                        + txarh.fillNColsFieldWithValue(df.format(ap.getImporte()), cols2[4])
                        + txarh.fillNColsFieldWithValue(dh, cols2[5]) + txarh.breakLine();


            }
            s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",addupcols2),addupcols2) + txarh.breakLine();

        }



        return s;
    }


    public static String getCaptionTipoPorcentajeMIVA(Double d){
        NumberFormat percent = NumberFormat.getPercentInstance();
        return percent.format(d);
    }

    public static String getCaptionTitular(String s){
        return s;
    }

    public static String getCaptionTipoPorcentajeMIRPF(MovtoIrpf mirpf){
        NumberFormat percent = NumberFormat.getPercentInstance();
        return percent.format(mirpf.getTipoPorcentaje());
    }


    public static String getCaption(CuentaContable cc){
        int y = 2;
        return cc.getCodigo() + " " + cc.getNombre();
    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        asientoDc.getItem().setFuente("UI");
        if (((User)currentAuthentication.getUser()).getEjercicioActivo()!=null){
            Ejercicio ej = ((User) currentAuthentication.getUser()).getEjercicioActivo();
            if (asientoDc.getItem().getEjercicio()==null){
                asientoDc.getItem().setEjercicio(ej);
            }

        }

        //if asiento is a new instance append 5 apuntes not initialized on each side of the asiento structure
        if (entityStates.isNew(asientoDc.getItem())){
            for (int i = 0; i < 5; i++) {
                Apunte apunteDebe = dataContext.create(Apunte.class);
                Apunte apunteHaber = dataContext.create(Apunte.class);
                apunteDebe.setAsiento(asientoDc.getItem());
                apunteHaber.setAsiento(asientoDc.getItem());
                apuntesDebeDc.getMutableItems().add(apunteDebe);
                apuntesHaberDc.getMutableItems().add(apunteHaber);
            }
        }
        try {
            if (asientoDc.getItem().getNumAsiento()==null){
                Integer numAsto = asientoBean.getNuevoNumAsiento(asientoDc.getItem().getEjercicio());
                numAsientoField.setValue(numAsto);
            }

        }catch(Exception exc){
            this.setReadOnly(true);
        }
    }

    @Subscribe("addApteDebe")
    public void onAddApteDebeClick(Button.ClickEvent event) {
        Apunte ap = dataContext.create(Apunte.class);
        apuntesDebeDc.getMutableItems().add(ap);
    }

    @Subscribe("addApteHaber")
    public void onAddApteHaberClick(Button.ClickEvent event) {
        Apunte ap = dataContext.create(Apunte.class);
        apuntesHaberDc.getMutableItems().add(ap);
    }

    @Subscribe("rmvApteDebe")
    public void onRmvApteDebeClick(Button.ClickEvent event) {
        final Apunte apteDebe = apuntesTableDebe.getSingleSelected();
        if (apteDebe==null){
            notifications.create().withCaption("Seleccionar un apunte Debe").withDescription("Selección nula").show();
            return;
        }else{
            dialogs.createOptionDialog().withCaption("Confirmación").withMessage("¿Desea eliminar el apunte seleccionado")
                    .withActions(
                            new DialogAction(DialogAction.Type.YES).withCaption("Confirmar").withHandler(e->{
                                apuntesDebeDc.getMutableItems().remove(apteDebe);
                                dataContext.remove(apteDebe);
                                notifications.create().withCaption("Apunte Eliminado").show();
                            }),
                            new DialogAction(DialogAction.Type.NO).withCaption("Cancelar").withHandler(e->{

                            })

                    ).show();

            return;
        }
    }

    @Subscribe("rmvApteHaber")
    public void onRmvApteHaberClick(Button.ClickEvent event) {
        final Apunte apteHaber = apuntesTableHaber.getSingleSelected();
        if (apteHaber==null){
            notifications.create().withCaption("Seleccionar un apunte Haber").withDescription("Selección nula").show();
            return;
        }else{
            dialogs.createOptionDialog().withCaption("Confirmación").withMessage("¿Desea eliminar el apunte seleccionado")
                    .withActions(
                            new DialogAction(DialogAction.Type.YES).withCaption("Confirmar").withHandler(e->{
                                apuntesHaberDc.getMutableItems().remove(apteHaber);
                                dataContext.remove(apteHaber);
                                notifications.create().withCaption("Apunte Eliminado").show();
                                apuntesTableHaber.deselectAll();
                                apuntesTableHaber.repaint();
                                
                            }),
                            new DialogAction(DialogAction.Type.NO).withCaption("Cancelar").withHandler(e->{

                            })

                    ).show();

            return;
        }
    }





    @Subscribe("addMovtoIva")
    public void onAddMovtoIvaClick(Button.ClickEvent event) {
        MovtoIva miva = dataContext.create(MovtoIva.class);
        movtoIvasDc.getMutableItems().add(miva);
    }

    @Subscribe("rmvMovtoIva")
    public void onRmvMovtoIvaClick(Button.ClickEvent event) {
        final MovtoIva miva = movtoIvasTable.getSingleSelected();
        if (miva==null){
            notifications.create().withCaption("Seleccionar un movimiento de IVA").withDescription("Selección nula").show();
            return;
        }else{
            dialogs.createOptionDialog().withCaption("Confirmación").withMessage("¿Desea eliminar el movimiento seleccionado")
                    .withActions(
                            new DialogAction(DialogAction.Type.YES).withCaption("Confirmar").withHandler(e->{
                                movtoIvasDc.getMutableItems().remove(miva);
                                dataContext.remove(miva);
                                notifications.create().withCaption("Movimiento Eliminado").show();
                            }),
                            new DialogAction(DialogAction.Type.NO).withCaption("Cancelar").withHandler(e->{

                            })

                    ).show();

            return;
        }
    }





    @Subscribe("addMovtoIrpf")
    public void onAddMovtoIrpfClick(Button.ClickEvent event) {
        MovtoIrpf mirpf = dataContext.create(MovtoIrpf.class);
        movtoIrpfsDc.getMutableItems().add(mirpf);
    }

    @Subscribe("rmvMovtoIrpf")
    public void onRmvMovtoIrpfClick(Button.ClickEvent event) {
        final MovtoIrpf mirpf = movtoIrpfsTable.getSingleSelected();
        if (mirpf==null){
            notifications.create().withCaption("Seleccionar un movimiento de Irpf").withDescription("Selección nula").show();
            return;
        }else{
            dialogs.createOptionDialog().withCaption("Confirmación").withMessage("¿Desea eliminar el apunte seleccionado")
                            .withActions(
                                    new DialogAction(DialogAction.Type.YES).withCaption("Confirmar").withHandler(e->{
                                        movtoIrpfsDc.getMutableItems().remove(mirpf);
                                        dataContext.remove(mirpf);
                                        notifications.create().withCaption("Movimiento Eliminado").show();
                                    }),
                                    new DialogAction(DialogAction.Type.NO).withCaption("Cancelar").withHandler(e->{

                                    })

                            ).show();

            return;
        }
    }

    @Install(to = "apuntesDebeDl", target = Target.DATA_LOADER)
    private List<Apunte> apuntesDebeDlLoadDelegate(LoadContext<Apunte> loadContext) {
        Asiento a = this.getEditedEntity();
        if (a!=null) {
            ArrayList<Apunte> aadd = new ArrayList<Apunte>();
            for (int i = 0; i < a.getApuntes().size(); i++) {
                Apunte ap = a.getApuntes().get(i);
                if (ap.getDebeHaber()) {
                    aadd.add(ap);
                }
            }
            return aadd;
        }else{
            return new ArrayList<>();
        }
    }

    @Install(to = "apuntesHaberDl", target = Target.DATA_LOADER)
    private List<Apunte> apuntesHaberDlLoadDelegate(LoadContext<Apunte> loadContext) {
        Asiento a = this.getEditedEntity();
        if (a!=null) {
            ArrayList<Apunte> aahh = new ArrayList<Apunte>();
            for (int i = 0; i < a.getApuntes().size(); i++) {
                Apunte ap = a.getApuntes().get(i);
                if (!ap.getDebeHaber()) {
                    aahh.add(ap);
                }
            }
            return aahh;
        }else{
            return new ArrayList<>();
        }
    }
    
    
    

    @Subscribe
    public void onBeforeCommitChanges(BeforeCommitChangesEvent event) {
        limpiaApuntesInterfazAsiento();

        //actualizando el miembro apuntes de asiento
        List<Apunte> apuntes = asientoDc.getItem().getApuntes();
        for (int i = 0; i < apuntesDebeDc.getItems().size(); i++) {
            Apunte ap = apuntesDebeDc.getItems().get(i);
            if (apuntes.indexOf(ap)==-1){
                apuntes.add(ap);
            }
        }
        for (int i = 0; i < apuntesHaberDc.getItems().size(); i++) {
            Apunte ap = apuntesHaberDc.getItems().get(i);
            if (apuntes.indexOf(ap)==-1){
                apuntes.add(ap);
            }
        }

        for (int i = 0; i < apuntes.size(); i++) {
            Apunte ap = apuntes.get(i);
            dataContext.merge(ap);
        }
    }

    @Subscribe("commitAndCloseBtn")
    public void onCommitAndCloseBtnClick(Button.ClickEvent event) {
        cancelDataGridEditors(apuntesTableDebe);
        cancelDataGridEditors(apuntesTableHaber);
        limpiaApuntesInterfazAsiento();
    }



    private void limpiaApuntesInterfazAsiento(){
        //adecuamos la lista de apuntes
        //si un apunte tiene cuenta e importe null lo eliminamos
        List<Apunte> todelete = new ArrayList();
        for (int i = 0; i < apuntesDebeDc.getItems().size(); i++) {
            Apunte ap = apuntesDebeDc.getItems().get(i);
            if ((ap.getCuentaContable()==null) && (ap.getImporte()==null)){
                todelete.add(ap);
            }
        }
        apuntesDebeDc.getMutableItems().removeAll(todelete);

        todelete = new ArrayList();
        for (int i = 0; i < apuntesHaberDc.getItems().size(); i++) {
            Apunte ap = apuntesHaberDc.getItems().get(i);
            if ((ap.getCuentaContable()==null) && (ap.getImporte()==null)){
                todelete.add(ap);
            }
        }
        apuntesHaberDc.getMutableItems().removeAll(todelete);
    }


    private void cancelDataGridEditors(DataGrid dataGrid){
        dataGrid.withUnwrapped(JmixGrid.class, jmixGrid -> {
            jmixGrid.getEditor().cancel();
        });
    }







    public List<CuentaContable> getCuentasContables(){
        String eql = "select cc from contabi_CuentaContable cc join cc.planContable pc where " +
                "pc.id = :pcid order by cc.codigo asc";
        List l = dataManager.load(CuentaContable.class).query(eql)
                .parameter("pcid", ((User) currentAuthentication.getUser()).getEjercicioActivo().getPlanContable().getId() )
                .list();
        return l;
    }

    public List<String> getTitulares(){
        String eql = "select t from contabi_Titular t order by t.nombre asc";
        List l = dataManager.load(Titular.class).query(eql)
                .list();
        ArrayList al = new ArrayList();
        for (int i = 0; i < l.size(); i++) {
            Titular t = (Titular)l.get(i);
            al.add(t.getNif() + " - " + t.getNombre());
        }
        return al;
    }


}