package contabilidad.screen.balancesituacion;

import contabilidad.beans.SaldosBean;

import contabilidad.beans.Txarh;
import contabilidad.entity.*;
import io.jmix.core.DataManager;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.reports.ReportImportExport;
import io.jmix.reports.ReportsPersistence;
import io.jmix.reports.entity.Report;
import io.jmix.reports.runner.ReportRunner;
import io.jmix.reports.util.ReportsUtils;
import io.jmix.reportsui.screen.report.wizard.ReportsWizard;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.*;
import io.jmix.ui.download.Downloader;
import io.jmix.ui.screen.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@UiController("contabi_BalanceSituacion")
@UiDescriptor("balance-situacion.xml")
@MultipleOpen
public class BalanceSituacion extends Screen {

    @Autowired
    private Button btnVer;
    @Autowired
    private ComboBox<Integer> cboProfundidad;
    @Autowired
    private EntityPicker<Ejercicio> pkrEjercicio;
    @Autowired
    private TextArea txaBalance;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private SaldosBean saldosBean;
    @Autowired
    private Notifications notifications;
    @Autowired
    private Txarh txarh;
    @Autowired
    private CheckBox chbCuentasSaldosCero;
    @Autowired
    private Button btnPdf;
    @Autowired
    private ReportRunner reportRunner;
    @Autowired
    private ReportsUtils reportsUtils;
    @Autowired
    private ReportsPersistence reportsPersistence;
    @Autowired
    private ReportImportExport reportImportExport;
    @Autowired
    private ReportsWizard reportsWizard;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private Downloader downloader;

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        pkrEjercicio.setValue(((User)currentAuthentication.getUser()).getEjercicioActivo());
        PlanContable pc = ((User) currentAuthentication.getUser()).getEjercicioActivo().getPlanContable();
        updateOptionsProfundidad(pc);
        chbCuentasSaldosCero.setValue(false);

    }

    @Subscribe
    public void onAfterInit(AfterInitEvent event) {
        btnVer.addClickListener(e->{
            try {
                Hashtable<CuentaContable, Double[]> balance = saldosBean.getBalanceAFecha(pkrEjercicio.getValue().getFechaFin(), pkrEjercicio.getValue(), pkrEjercicio.getValue().getPlanContable(), cboProfundidad.getValue());
                if (chbCuentasSaldosCero.getValue()==false){
                    balance = saldosBean.eliminaCuentasSaldoCero(balance);
                }
                String s = getTxaReportBalanceSituacion(balance, pkrEjercicio.getValue().getFechaFin(), pkrEjercicio.getValue().getEmpresa(), pkrEjercicio.getValue(), txarh);
                txaBalance.setValue(s);
            } catch (Exception ex) {
                notifications.create().withDescription(ex.getMessage()).show();
            }
        });

        btnPdf.addClickListener(e->{
           try{
               Hashtable<CuentaContable, Double[]> balance = saldosBean.getBalanceAFecha(pkrEjercicio.getValue().getFechaFin(), pkrEjercicio.getValue(), pkrEjercicio.getValue().getPlanContable(), cboProfundidad.getValue());
               if (chbCuentasSaldosCero.getValue()==false){
                   balance = saldosBean.eliminaCuentasSaldoCero(balance);
               }
               Double[] saldos = saldosBean.getSumaDebeHaberDeBalance(balance);

               String caracteristicas = "Balance de Situación";
               caracteristicas += "Profundidad: " + cboProfundidad.getValue().toString();
               List<PartidaBalance> ppbb = saldosBean.transformaBalance(balance);
               HashMap<String, Object> params = new HashMap<>();
               params.put("partidas", ppbb);
               params.put("empresa", pkrEjercicio.getValue().getEmpresa());
               params.put("ejercicio", pkrEjercicio.getValue());
               params.put("tituloDocumento", "Balance de Situación");
               params.put("caracteristicasDocumento", caracteristicas);
               params.put("totalDebe", saldos[0]);
               params.put("totalHaber", saldos[1]);
               Report r = dataManager.load(Report.class).query("select r from report_Report r where r.name like :rn")
                       .parameter("rn", "Balance").one();
               byte[] bb = reportRunner.byReportEntity(r).withParams(params).run().getContent();
               downloader.download(bb, "Balance.pdf");
               int y = 2;
           }catch(Exception exc){
               notifications.create().withDescription(exc.getMessage()).show();
           }
        });
    }


    public static String getTxaReportBalanceSituacion(Hashtable<CuentaContable, Double[]> balance, Date fechaCalculo, Empresa emp, Ejercicio ej, Txarh txarh) throws Exception{

        ArrayList<CuentaContable> cuentas = new ArrayList(balance.keySet());
        Collections.sort(cuentas, txarh.comparatorCC);

        String s = "";
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm");
        SimpleDateFormat sdf2 = new SimpleDateFormat("dd/MM/yyyy");
        DecimalFormat df = new DecimalFormat("#,##0.00");
        //cabeza fecha y hora
        s += txarh.fillNColsFieldWithValue(sdf.format(new Date()), 40) + txarh.breakLine();
        //nombre empresa
        s += txarh.fillNColsFieldWithValue(emp.getNombre(), 50) + txarh.fillNColsFieldWithValue("(Ejercicio  " + ej.getAnno().toString() + ")", 30, txarh.HORIZONTAL_ALIGNMENT_RIGHT) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Informe Balance de Sumas y Saldos", 70) + txarh.fillNColsFieldWithValue("(Nivel Subcuenta)", 30) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Ordenado Por      : Subcuenta", 60) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Fecha Cálculo:    " + sdf.format(fechaCalculo), 70) + txarh.breakLine();
        //separadores
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",50),50) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Límites Impuestos:      Sin Limitación", 60) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",50),50) + txarh.breakLine();
        s += txarh.breakLine() + txarh.breakLine();

        int[] cols = new int[]{
                15, //codigo
                60, //nombre cuenta
                15, //ac. debe
                15, //ac. haber
                30, //saldo
        };
        int acccolumns = 0;
        for (int i = 0; i < cols.length; i++) {
            acccolumns += cols[i];
        }
        //columnas
        s += txarh.fillNColsFieldWithValue("Código", cols[0]) +
                txarh.fillNColsFieldWithValue("Nombre", cols[1]) +
                txarh.fillNColsFieldWithValue("Ac.Debe", cols[2], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                txarh.fillNColsFieldWithValue("Ac.Haber", cols[3], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                txarh.fillNColsFieldWithValue("Saldo", cols[4], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",acccolumns),acccolumns) + txarh.breakLine();
        //cuentas
        double totalSaldoDebe = 0.0, totalSaldoHaber = 0.0;
        for (int i = 0; i < cuentas.size(); i++) {
            CuentaContable cc = cuentas.get(i);
            Double[] saldos = balance.get(cc);
            totalSaldoDebe += saldos[0];
            totalSaldoHaber += saldos[1];
            String txtDH = "";
            if (saldos[0]>saldos[1]){
                txtDH = " D ";
            }else if (saldos[1]>saldos[0]){
                txtDH = " H ";
            }

            s += txarh.fillNColsFieldWithValue(cc.getCodigo(), cols[0]) +
                    txarh.fillNColsFieldWithValue(cc.getNombre(), cols[1]) +
                    txarh.fillNColsFieldWithValue(df.format(saldos[0]), cols[2], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                    txarh.fillNColsFieldWithValue(df.format(saldos[1]), cols[3], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                    txarh.fillNColsFieldWithValue(df.format(Math.abs(saldos[2])) + txtDH, cols[4], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                    txarh.breakLine();
        }
        s += StringUtils.repeat("-", acccolumns) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Totales", cols[0]) +
                StringUtils.repeat(" ", cols[1]) +
                txarh.fillNColsFieldWithValue(df.format(totalSaldoDebe), cols[2], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                txarh.fillNColsFieldWithValue(df.format(totalSaldoHaber), cols[3], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                txarh.fillNColsFieldWithValue(df.format(Math.abs(totalSaldoDebe - totalSaldoHaber)) + " D ", cols[4], Txarh.HORIZONTAL_ALIGNMENT_RIGHT) +
                txarh.breakLine();


        return s;
    }



    @Subscribe("pkrEjercicio")
    public void onPkrEjercicioValueChange(HasValue.ValueChangeEvent<Ejercicio> event) {
        PlanContable pc = event.getValue().getPlanContable();
        updateOptionsProfundidad(pc);
        txaBalance.setValue("");
    }



    private void updateOptionsProfundidad(PlanContable pc){

        ArrayList<Integer> options = new ArrayList<Integer>();
        for (int i = 0; i < pc.getProfundidad(); i++) {
            options.add(i+1);
        }
        cboProfundidad.setOptionsList(options);
        cboProfundidad.setValue(options.get(options.size()-1));
    }



}