package contabilidad.screen.centrodecoste;

import contabilidad.entity.Apunte;
import contabilidad.entity.Asiento;
import contabilidad.entity.CentroDeCoste;
import io.jmix.core.DataManager;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.UiComponents;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.FlowBoxLayout;
import io.jmix.ui.component.HtmlBoxLayout;
import io.jmix.ui.component.TextField;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.*;
import contabilidad.entity.ImputacionCentroDeCoste;
import org.aspectj.apache.bcel.classfile.Module;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

@UiController("contabi_ImputacionCentroDeCoste.edit")
@UiDescriptor("imputacion-centro-de-coste-edit.xml")
@EditedEntityContainer("imputacionCentroDeCosteDc")
public class ImputacionCentroDeCosteEdit extends StandardEditor<ImputacionCentroDeCoste> {

    @Autowired
    private InstanceContainer<ImputacionCentroDeCoste> imputacionCentroDeCosteDc;
    @Autowired
    private Button btnCentroDeCoste;
    @Autowired
    private ScreenBuilders screenBuilders;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private UiComponents uiComponents;
    @Autowired
    private FlowBoxLayout buttonsImportes;
    @Autowired
    private TextField<Double> importeConSignoField;

    public void setAsiento(Asiento a){
        imputacionCentroDeCosteDc.getItem().setAsiento(a);
        this.cargaBotonesAsiento();
    }

    @Subscribe("btnCentroDeCoste")
    public void onBtnCentroDeCosteClick(Button.ClickEvent event) {
        SeleccionCentroCoste s = (SeleccionCentroCoste) screenBuilders.screen(this).withScreenId("contabi_SeleccionCentroCoste").withOpenMode(OpenMode.DIALOG)
                .build();
        s.addAfterShowListener(e->{
           s.setPlanContable(this.getEditedEntity().getAsiento().getEjercicio().getPlanContable());
        });
        s.addAfterCloseListener(e->{
            CentroDeCoste cc = s.getSeleccion();
            if (cc!=null){
                imputacionCentroDeCosteDc.getItem().setCentroDeCoste(cc);
                setNombreButtonCentroDeCoste(cc);
            }
        });
        s.show();
    }

    private void setNombreButtonCentroDeCoste(CentroDeCoste cc) {
        //recargando nombre hasta root
        String bc = cc.getNombre();
        while(true){
            cc = dataManager.load(CentroDeCoste.class).id(cc.getId()).fetchPlan("fp-centro-de-coste-recursivo").one();
            if (cc.getCentroDeCosteAntecesor()!=null){
                cc = cc.getCentroDeCosteAntecesor();
                bc = cc.getNombre() + "/" + bc;
            }else{
                break;
            }
            btnCentroDeCoste.setCaption(bc);
        }
    }
    
    
    

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        
        if (this.getEditedEntity().getCentroDeCoste()==null){
            btnCentroDeCoste.setCaption("(Seleccionar Centro de Coste)");
        }else{
            setNombreButtonCentroDeCoste(this.getEditedEntity().getCentroDeCoste());
        }

    }

    private void cargaBotonesAsiento(){
        //cargando los botones de importes
        List<Apunte> apuntes = imputacionCentroDeCosteDc.getItem().getAsiento().getApuntes();
        for (int i = 0; i < apuntes.size(); i++) {
            Apunte ap = apuntes.get(i);
            Button b = uiComponents.create(Button.NAME);
            b.setCaption(ap.getImporte().toString());
            b.addClickListener(e->{
                importeConSignoField.setValue(ap.getImporte());
            });
            buttonsImportes.add(b);
        }
    }
    
    

}