package contabilidad.screen.extractosimple;

import contabilidad.beans.SaldosBean;

import contabilidad.beans.Txarh;
import contabilidad.entity.*;
import contabilidad.screen.asiento.AsientoEdit;
import contabilidad.screen.asiento.AsientoEdit2;
import io.jmix.core.DataManager;
import io.jmix.core.FetchPlan;
import io.jmix.core.Metadata;
import io.jmix.core.metamodel.model.MetaClass;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.reports.entity.Report;
import io.jmix.reports.runner.ReportRunner;
import io.jmix.ui.Notifications;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.UiComponents;
import io.jmix.ui.component.*;
import io.jmix.ui.component.data.DataGridItems;
import io.jmix.ui.component.data.datagrid.ContainerDataGridItems;
import io.jmix.ui.download.Downloader;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.model.impl.CollectionContainerImpl;
import io.jmix.ui.screen.*;
import org.apache.commons.lang3.StringUtils;

import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.lang.reflect.Array;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@UiController("contabi_ExtractoSimple")
@UiDescriptor("extracto-simple.xml")
@MultipleOpen
public class ExtractoSimple extends Screen {

    @Autowired
    private CollectionContainer<Apunte> apuntesDc;
    @Autowired
    private ComboBox<CuentaContable> cboCuentaDesde;
    @Autowired
    private ComboBox<CuentaContable> cboCuentaHasta;
    @Autowired
    private Button btnGo;
    @Autowired
    private Button btnAlternarTxaDataGrid;
    @Autowired
    private TextArea txa;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private DateField<Date> afecha;
    @Autowired
    private DateField<Date> defecha;
    @Autowired
    private Txarh txarh;
    @Autowired
    private Notifications notifications;
    @Autowired
    private SaldosBean saldosBean;
    @Autowired
    private Button btnPdf;
    @Autowired
    private ReportRunner reportRunner;
    @Autowired
    private Downloader downloader;
    @Autowired
    private TagField<CuentaContable> tgCuentas;
    @Autowired
    private HBoxLayout hbxModoTag;
    @Autowired
    private HBoxLayout hbxModoDeA;
    @Autowired
    private ComboBox<String> cboModoSeleccionCuentas;
    @Autowired
    private UiComponents uiComponents;
    @Autowired
    private Metadata metadata;
    @Autowired
    private VBoxLayout dataGridsContainer;
    @Autowired
    private VBoxLayout extractoContainer;
    @Autowired
    private ScreenBuilders screenBuilders;

    @Install(to = "tgCuentas", subject = "searchExecutor")
    private List<CuentaContable> tgCuentasSearchExecutor(String searchString, Map<String, Object> searchParams) {
        Ejercicio ej = ((User)currentAuthentication.getUser()).getEjercicioActivo();
        PlanContable pc = ej.getPlanContable();

        String eql = "select cc from contabi_CuentaContable cc join cc.planContable pc where pc.id = :pcid and concat(cc.codigo,cc.nombre) like :ss";
        List<CuentaContable> l = dataManager.load(CuentaContable.class).query(eql)
                .parameter("pcid", pc.getId())
                .parameter("ss","%"+searchString+"%")
                .list();
        Collections.sort(l, txarh.comparatorCC);
        return l;
    }

    private DataGrid getDataGridForCuentaApuntes(CuentaContable cc, List<Apunte> apuntes){
        MetaClass apunteMetaClass = metadata.getClass(Apunte.class);
        CollectionContainerImpl cci = new CollectionContainerImpl(apunteMetaClass);
        cci.setItems(apuntes);
        DataGrid dg = uiComponents.create(DataGrid.NAME);
        DataGridItems<Apunte> dgi = new ContainerDataGridItems<>(cci);
        dg.setItems(dgi);
        ArrayList<DataGrid.Column> cols = new ArrayList(dg.getColumns());
        for (int i = 0; i < cols.size(); i++) {
            DataGrid.Column c = cols.get(i);
            dg.removeColumn(c.getId());
        }

        dg.addColumn("fecha", apunteMetaClass.getPropertyPath("asiento.fecha"));
        dg.addColumn("ampliacion", apunteMetaClass.getPropertyPath("ampliacion"));
        dg.addColumn("debeHaber", apunteMetaClass.getPropertyPath("debeHaber"));
        dg.addColumn("importe", apunteMetaClass.getPropertyPath("importe"));
        dg.addColumn("numAsiento", apunteMetaClass.getPropertyPath("asiento.numAsiento"));
        DataGrid.ButtonRenderer<Apunte> numAsientoRenderer =
                getApplicationContext().getBean(DataGrid.ButtonRenderer.class);
        numAsientoRenderer.setRendererClickListener(e->{
                Asiento as = e.getItem().getAsiento();
                AsientoEdit2 ae2scr = (AsientoEdit2)screenBuilders.editor(Asiento.class, this).withScreenId("contabi_Asiento.edit2")
                        .editEntity(as).withOpenMode(OpenMode.NEW_TAB).build();
                ae2scr.show();
        });
        dg.getColumn("numAsiento")
                .setRenderer(numAsientoRenderer);
        dg.addColumn("numApunte", apunteMetaClass.getPropertyPath("numApunte"));


        return dg;
    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        extractoContainer.setExpandRatio(txa, 100.0f);
    }




    @Subscribe
    public void onAfterInit(AfterInitEvent event) {

        cboModoSeleccionCuentas.setOptionsList(Arrays.asList(new String[]{"De/A","Discontinuo"}));

        cboModoSeleccionCuentas.addValueChangeListener(e->{
            if (e.getValue().compareTo("De/A")==0){
                hbxModoDeA.setVisible(true);
                hbxModoTag.setVisible(false);
            }else{
                hbxModoDeA.setVisible(false);
                hbxModoTag.setVisible(true);
            }
        });
        cboModoSeleccionCuentas.setValue("De/A");

        cboCuentaDesde.setPopupWidth("500px");
        cboCuentaDesde.setPageLength(10);
        cboCuentaDesde.setOptionCaptionProvider(ExtractoSimple::getCaption);
        cboCuentaDesde.setOptionsList(getCuentasContables());
        cboCuentaDesde.setFilterMode(HasFilterMode.FilterMode.CONTAINS);

        cboCuentaDesde.addValueChangeListener(e->{
           CuentaContable cc = e.getValue();
           cboCuentaHasta.setValue(cc);
        });

        cboCuentaHasta.setPopupWidth("500px");
        cboCuentaHasta.setPageLength(10);
        cboCuentaHasta.setOptionCaptionProvider(ExtractoSimple::getCaption);
        cboCuentaHasta.setOptionsList(getCuentasContables());
        cboCuentaHasta.setFilterMode(HasFilterMode.FilterMode.CONTAINS);

        if (((User)currentAuthentication.getUser()).getEjercicioActivo()!=null){
            Ejercicio ej = ((User)currentAuthentication.getUser()).getEjercicioActivo();
            defecha.setValue(ej.getFechaInicio());
            afecha.setValue(ej.getFechaFin());
        }

        btnGo.addClickListener(e->{

            Ejercicio ej = ((User)currentAuthentication.getUser()).getEjercicioActivo();
            List<Apunte> aps = null;
            if (cboModoSeleccionCuentas.getValue().compareTo("De/A")==0){
                aps = saldosBean.getApuntes(cboCuentaDesde.getValue().getCodigo(),
                        cboCuentaHasta.getValue().getCodigo(), (Date)defecha.getValue(), (Date)afecha.getValue(), ej.getEmpresa());
            }else{
                List<CuentaContable> ccs = new ArrayList<>(tgCuentas.getValue());
                ArrayList<String> al = new ArrayList();
                for (int i = 0; i < ccs.size(); i++) {
                    al.add(ccs.get(i).getCodigo());
                }
                String[] codigos = al.toArray(new String[0]);
                aps = saldosBean.getApuntes(codigos, (Date)defecha.getValue(), (Date)afecha.getValue(), ej.getEmpresa());
            }


           Hashtable<CuentaContable, List<Apunte>> cuentasApuntes = new Hashtable<CuentaContable, List<Apunte>>();
            for (int i = 0; i < aps.size(); i++) {
                Apunte ap = aps.get(i);
                if (!cuentasApuntes.containsKey(ap.getCuentaContable())){
                    cuentasApuntes.put(ap.getCuentaContable(), new ArrayList<Apunte>());
                }
                List l = cuentasApuntes.get(ap.getCuentaContable());
                l.add(ap);
            }

            try {
                String s = "";
                if (cboModoSeleccionCuentas.getValue().compareTo("De/A")==0){
                    CuentaContable ccDesde = cboCuentaDesde.getValue();
                    CuentaContable ccHasta = cboCuentaHasta.getValue();
                    Date desdeFecha = defecha.getValue();
                    Date aFecha = afecha.getValue();
                    s = getTxaReportExtracto(ccDesde, ccHasta, desdeFecha, aFecha, ej);
                }else{
                    ArrayList<CuentaContable> ccs = new ArrayList(tgCuentas.getValue());
                    Date desdeFecha = defecha.getValue();
                    Date aFecha = afecha.getValue();
                    s = getTxaReportExtracto(ccs, desdeFecha, aFecha, ej);

                }



                txa.setValue(s);
            } catch (Exception ex) {
                notifications.create().withDescription(ex.getMessage()).show();
            }

        });

        btnPdf.addClickListener(e->{
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yy");
            Ejercicio ej = ((User)currentAuthentication.getUser()).getEjercicioActivo();
            List<Apunte> aps = saldosBean.getApuntes(cboCuentaDesde.getValue().getCodigo(),
                    cboCuentaHasta.getValue().getCodigo(), (Date)defecha.getValue(), (Date)afecha.getValue(), ej.getEmpresa());

            Hashtable<CuentaContable, List<Apunte>> cuentasApuntes = new Hashtable<CuentaContable, List<Apunte>>();
            for (int i = 0; i < aps.size(); i++) {
                Apunte ap = aps.get(i);
                if (!cuentasApuntes.containsKey(ap.getCuentaContable())){
                    cuentasApuntes.put(ap.getCuentaContable(), new ArrayList<Apunte>());
                }
                List l = cuentasApuntes.get(ap.getCuentaContable());
                l.add(ap);
            }
            if(cuentasApuntes.keySet().size()>1){
                notifications.create().withDescription("Este report puede producir reports de momento tan " +
                        "solo de una cuenta. Por favor ajustar con una única cuenta").show();
                return;
            }else if(cuentasApuntes.keySet().size()==1){
                CuentaContable cc = (CuentaContable) new ArrayList(cuentasApuntes.keySet()).get(0);
                ArrayList apuntes = (ArrayList) cuentasApuntes.get(cc);
                Collections.sort(apuntes, new Comparator<Apunte>() {
                    @Override
                    public int compare(Apunte o1, Apunte o2) {
                        return o1.getAsiento().getFecha().compareTo(o2.getAsiento().getFecha());
                    }
                });
                double[] saldos = null;
                try{
                    saldos = saldosBean.getSaldosCuentaAFecha(cc, defecha.getValue());
                }catch(Exception exc){
                    notifications.create().withDescription("No se pudo crear el informe de extracto:" + exc.getMessage()).show();
                }

                ArrayList partidasExtracto = new ArrayList();
                double saldo = saldos[0] - saldos[1];
                String saldoInicialDH="D";
                String saldoFinalDH = "D";

                double saldoInicial = saldo;
                if (saldoInicial>=0) saldoInicialDH = "D"; else saldoFinalDH = "H";


                for (int i = 0; i < apuntes.size(); i++) {
                    Apunte ap = (Apunte) apuntes.get(i);
                    PartidaExtracto pe = new PartidaExtracto();
                    if (ap.getConceptoApunte()!=null){
                        pe.setConcepto(ap.getConceptoApunte().getCodigo());
                    }else{
                        pe.setConcepto("");
                    }

                    pe.setAmpliacion(ap.getAmpliacion());
                    pe.setFecha(sdf.format(ap.getAsiento().getFecha()));
                    pe.setNumApunte(ap.getNumApunte());
                    pe.setNumAsiento(ap.getAsiento().getNumAsiento());
                    if (ap.getDebeHaber()){
                        pe.setDebe(ap.getImporte());
                        pe.setHaber(0.0);
                        saldo += ap.getImporte();
                    }else{
                        pe.setHaber(ap.getImporte());
                        pe.setDebe(0.0);
                        saldo -= ap.getImporte();
                    }
                    if (saldo>=0.0){
                        pe.setSaldoDebeHaber("D");
                    }else{
                        pe.setSaldoDebeHaber("H");
                    }
                    pe.setSaldo(Math.abs(saldo));
                    partidasExtracto.add(pe);
                }
                double saldoFinal = saldo;
                if (saldoFinal>=0) saldoFinalDH = "D";else saldoFinalDH = "H";

                Hashtable<String, Object> params = new Hashtable<>();
                params.put("fechaRealizacion", new Date());
                params.put("partidasExtracto", partidasExtracto);
                params.put("empresa", cc.getEmpresa());
                params.put("saldoInicial", saldoInicial);
                params.put("saldoInicialDH", saldoInicialDH);
                params.put("saldoFinal", saldoFinal);
                params.put("saldoFinalDH", saldoFinalDH);
                params.put("titulo", "Extracto Simple de Cuenta");
                params.put("codigoCuentaContable", cc.getCodigo() + " - " + cc.getNombre());
                params.put("periodoFechaDesde", defecha.getValue());
                params.put("periodoFechaHasta", afecha.getValue());

                Report r = dataManager.load(Report.class).query("select r from report_Report r where r.name like :rn")
                        .parameter("rn", "Extracto Simple 1 Cuenta").one();
                byte[] bb = reportRunner.byReportEntity(r).withParams(params).run().getContent();
                downloader.download(bb, "Extracto Cuenta Simple " + cc.getCodigo() + " " + cc.getNombre() + ".pdf");
            }

        });



        btnAlternarTxaDataGrid.addClickListener(e->{
            if (extractoContainer.getExpandRatio(txa)==100.0f){

                extractoContainer.setExpandRatio(txa, 0.0f);
                extractoContainer.setExpandRatio(dataGridsContainer, 100.0f);

                Ejercicio ej = ((User)currentAuthentication.getUser()).getEjercicioActivo();
                List<Apunte> aps = null;
                if (cboModoSeleccionCuentas.getValue().compareTo("De/A")==0){
                    aps = saldosBean.getApuntes(cboCuentaDesde.getValue().getCodigo(),
                            cboCuentaHasta.getValue().getCodigo(), (Date)defecha.getValue(), (Date)afecha.getValue(), ej.getEmpresa());
                }else{
                    List<CuentaContable> ccs = new ArrayList<>(tgCuentas.getValue());
                    ArrayList<String> al = new ArrayList();
                    for (int i = 0; i < ccs.size(); i++) {
                        al.add(ccs.get(i).getCodigo());
                    }
                    String[] codigos = al.toArray(new String[0]);
                    aps = saldosBean.getApuntes(codigos, (Date)defecha.getValue(), (Date)afecha.getValue(), ej.getEmpresa());
                }


                Hashtable<CuentaContable, List<Apunte>> cuentasApuntes = new Hashtable<CuentaContable, List<Apunte>>();
                for (int i = 0; i < aps.size(); i++) {
                    Apunte ap = aps.get(i);
                    if (!cuentasApuntes.containsKey(ap.getCuentaContable())){
                        cuentasApuntes.put(ap.getCuentaContable(), new ArrayList<Apunte>());
                    }
                    List l = cuentasApuntes.get(ap.getCuentaContable());
                    l.add(ap);
                }

                Iterator<CuentaContable> itcc = cuentasApuntes.keySet().iterator();
                while(itcc.hasNext()){
                    CuentaContable cc = itcc.next();
                    List<Apunte> apuntes = cuentasApuntes.get(cc);
                    Collections.sort(apuntes, new Comparator<Apunte>(){

                        @Override
                        public int compare(Apunte o1, Apunte o2) {
                            return o1.getAsiento().getFecha().compareTo(o2.getAsiento().getFecha());
                        }
                    });
                    DataGrid dg = getDataGridForCuentaApuntes(cc, apuntes);
                    dg.setWidth("100%");
                    Label l = uiComponents.create(Label.NAME);

                    l.setValue(cc.getCodigo() + " - " + cc.getNombre());
                    dataGridsContainer.add(l);
                    dataGridsContainer.add(dg);
                    dataGridsContainer.setExpandRatio(l, 0.05f);
                    dataGridsContainer.setExpandRatio(dg, 0.95f);
                }

            }else{
                extractoContainer.setExpandRatio(txa, 100.0f);
                extractoContainer.setExpandRatio(dataGridsContainer, 0.0f);
                dataGridsContainer.removeAll();
            }
        });



    }

    private String getTxaReportExtracto(List<CuentaContable> ccs, Date defecha, Date afecha, Ejercicio ejActivo) throws Exception{

        String s = "";
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm");
        SimpleDateFormat sdf2 = new SimpleDateFormat("dd/MM/yyyy");
        DecimalFormat df = new DecimalFormat("#,##0.00");
        //cabeza fecha y hora
        s += txarh.fillNColsFieldWithValue(sdf.format(new Date()), 40) + txarh.breakLine();
        //nombre empresa
        CuentaContable ccDesde = ccs.get(0);
        s += txarh.fillNColsFieldWithValue(ccDesde.getEmpresa().getNombre(), 50) + txarh.fillNColsFieldWithValue("(Ejercicio  " + ejActivo.getAnno().toString() + ")", 30, txarh.HORIZONTAL_ALIGNMENT_RIGHT) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Extractos Generales", 30) + txarh.breakLine();
        //separador
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",100),100) + txarh.breakLine();
//        s += txarh.fillNColsFieldWithValue("Cuentas:", 40) + txarh.fillNColsFieldWithValue("Desde", 10) + txarh.fillNColsFieldWithValue("Hasta", 10) + txarh.breakLine();
//        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",100),100) + txarh.breakLine();
//        s += txarh.fillNColsFieldWithValue("Código", 40) + txarh.fillNColsFieldWithValue(ccDesde.getCodigo(), 15) + txarh.fillNColsFieldWithValue(ccHasta.getCodigo(), 15) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Fecha", 40) + txarh.fillNColsFieldWithValue(sdf2.format(defecha), 15) + txarh.fillNColsFieldWithValue(sdf2.format(afecha), 15) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",100),100);
        s += txarh.breakLine() + txarh.breakLine();


        List<CuentaContable> cuentas = ccs;
        Collections.sort(cuentas, txarh.comparatorCC);
        for (int i = 0; i < cuentas.size(); i++) {
            CuentaContable cc = cuentas.get(i);
            //subcuenta
            s += txarh.fillNColsFieldWithValue("Subcuenta     " + cc.getCodigo() + " - " + cc.getNombre(), 100) + txarh.breakLine();
            s += txarh.breakLine();
            //cabeceras
            int[] cols = new int[]{20,//fecha
                    40,//ampliacion
                    15,//debe
                    5,//espacio
                    15,//haber
                    5,//espacio
                    15,//saldo
                    5,//espacio
                    12};//asto/apte
            s += txarh.fillNColsFieldWithValue("Fecha", cols[0]) +
                    txarh.fillNColsFieldWithValue("Ampliación", cols[1]) +
                    txarh.fillNColsFieldWithValue("Debe", cols[2]) +
                    txarh.fillNColsFieldWithValue(" ", cols[3]) +
                    txarh.fillNColsFieldWithValue("Haber", cols[4]) +
                    txarh.fillNColsFieldWithValue(" ", cols[5]) +
                    txarh.fillNColsFieldWithValue("Saldo", cols[6]) +
                    txarh.fillNColsFieldWithValue(" ", cols[7]) +
                    txarh.fillNColsFieldWithValue("Asto/Apte", cols[8]) +
                    txarh.breakLine();
            s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-", cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]), cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]) + txarh.breakLine();
            //saldos

            double[] saldos = saldosBean.getSaldosCuentaAFecha(cc, defecha);
            double saldo = Math.abs(saldos[0] - saldos[1]);
            s += txarh.fillNColsFieldWithValue("S.Anterior", cols[0]+cols[1]) + txarh.fillNColsFieldWithValue(df.format(saldos[0]), cols[2]) + StringUtils.repeat(" ", cols[3]) + txarh.fillNColsFieldWithValue(df.format(saldos[1]), cols[4]) + txarh.fillNColsFieldWithValue(" ", cols[5]) + txarh.fillNColsFieldWithValue(df.format(saldo), cols[6]) + txarh.breakLine();
            //movimientos
            double saldoProv = saldos[0] - saldos[1];
            double saldoAccDebe = saldos[0];
            double saldoAccHaber = saldos[1];

            List<Apunte> apuntes = saldosBean.getApuntes(cc.getCodigo(), cc.getCodigo(), defecha, afecha, ejActivo.getEmpresa());
            Collections.sort(apuntes, txarh.comparatorAps);
            for (int j = 0; j < apuntes.size(); j++) {

                Apunte ap = apuntes.get(j);
                s += txarh.fillNColsFieldWithValue(sdf2.format(ap.getAsiento().getFecha()), cols[0]) +
                        txarh.fillNColsFieldWithValue(ap.getAmpliacion(), cols[1]);
                if (ap.getDebeHaber()){
                    saldoProv += ap.getImporte();
                    saldoAccDebe += ap.getImporte();
                    s += txarh.fillNColsFieldWithValue(df.format(ap.getImporte()),cols[2])
                            + StringUtils.repeat(" ", cols[3])
                            + txarh.fillNColsFieldWithValue("0.00", cols[4])
                            + StringUtils.repeat(" ", cols[5])
                            + txarh.fillNColsFieldWithValue(getTxtSaldoProv(saldoProv, df), cols[6])
                            + StringUtils.repeat(" ", cols[7])
                            + txarh.fillNColsFieldWithValue(ap.getAsiento().getNumAsiento().toString() + "/" + ap.getNumApunte(), cols[8])
                            + txarh.breakLine();

                }else{
                    saldoProv -= ap.getImporte();
                    saldoAccHaber += ap.getImporte();
                    s += txarh.fillNColsFieldWithValue("0.00",cols[2])
                            + StringUtils.repeat(" ", cols[3])
                            + txarh.fillNColsFieldWithValue(df.format(ap.getImporte()), cols[4])
                            + StringUtils.repeat(" ", cols[5])
                            + txarh.fillNColsFieldWithValue(getTxtSaldoProv(saldoProv, df), cols[6])
                            + StringUtils.repeat(" ", cols[7])
                            + txarh.fillNColsFieldWithValue(ap.getAsiento().getNumAsiento().toString() + "/" + ap.getNumApunte(), cols[8])
                            + txarh.breakLine();

                }

            }
            s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-", cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]), cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]) + txarh.breakLine();
            s += txarh.fillNColsFieldWithValue("Totales", cols[0]+cols[1]) +
                    txarh.fillNColsFieldWithValue(df.format(saldoAccDebe), cols[2]) +
                    txarh.fillNColsFieldWithValue(" ", cols[3]) +
                    txarh.fillNColsFieldWithValue(df.format(saldoAccHaber), cols[4]) +
                    txarh.fillNColsFieldWithValue(" ", cols[5]) +
                    txarh.fillNColsFieldWithValue(df.format(saldoProv), cols[6]) +
                    txarh.breakLine();
            s += txarh.breakLine();

        }


        return s;
    }

    private String getTxaReportExtracto(CuentaContable ccDesde, CuentaContable ccHasta, Date defecha, Date afecha, Ejercicio ejActivo) throws Exception{

        String s = "";
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm");
        SimpleDateFormat sdf2 = new SimpleDateFormat("dd/MM/yyyy");
        DecimalFormat df = new DecimalFormat("#,##0.00");
        //cabeza fecha y hora
        s += txarh.fillNColsFieldWithValue(sdf.format(new Date()), 40) + txarh.breakLine();
        //nombre empresa
        s += txarh.fillNColsFieldWithValue(ccDesde.getEmpresa().getNombre(), 50) + txarh.fillNColsFieldWithValue("(Ejercicio  " + ejActivo.getAnno().toString() + ")", 30, txarh.HORIZONTAL_ALIGNMENT_RIGHT) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Extractos Generales", 30) + txarh.breakLine();
        //separador
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",100),100) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Cuentas:", 40) + txarh.fillNColsFieldWithValue("Desde", 10) + txarh.fillNColsFieldWithValue("Hasta", 10) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",100),100) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Código", 40) + txarh.fillNColsFieldWithValue(ccDesde.getCodigo(), 15) + txarh.fillNColsFieldWithValue(ccHasta.getCodigo(), 15) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue("Fecha", 40) + txarh.fillNColsFieldWithValue(sdf2.format(defecha), 15) + txarh.fillNColsFieldWithValue(sdf2.format(afecha), 15) + txarh.breakLine();
        s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-",100),100);
        s += txarh.breakLine() + txarh.breakLine();


        List<CuentaContable> cuentas = saldosBean.devuelveListaDeCuentasDesdeHasta(ccDesde,ccHasta);
        Collections.sort(cuentas, txarh.comparatorCC);
        for (int i = 0; i < cuentas.size(); i++) {
            CuentaContable cc = cuentas.get(i);
            //subcuenta
            s += txarh.fillNColsFieldWithValue("Subcuenta     " + cc.getCodigo() + " - " + cc.getNombre(), 100) + txarh.breakLine();
            s += txarh.breakLine();
            //cabeceras
            int[] cols = new int[]{20,//fecha
                    40,//ampliacion
                    15,//debe
                    5,//espacio
                    15,//haber
                    5,//espacio
                    15,//saldo
                    5,//espacio
                    12};//asto/apte
            s += txarh.fillNColsFieldWithValue("Fecha", cols[0]) +
                    txarh.fillNColsFieldWithValue("Ampliación", cols[1]) +
                    txarh.fillNColsFieldWithValue("Debe", cols[2]) +
                    txarh.fillNColsFieldWithValue(" ", cols[3]) +
                    txarh.fillNColsFieldWithValue("Haber", cols[4]) +
                    txarh.fillNColsFieldWithValue(" ", cols[5]) +
                    txarh.fillNColsFieldWithValue("Saldo", cols[6]) +
                    txarh.fillNColsFieldWithValue(" ", cols[7]) +
                    txarh.fillNColsFieldWithValue("Asto/Apte", cols[8]) +
                    txarh.breakLine();
            s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-", cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]), cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]) + txarh.breakLine();
            //saldos

            double[] saldos = saldosBean.getSaldosCuentaAFecha(cc, defecha);
            double saldo = Math.abs(saldos[0] - saldos[1]);
            s += txarh.fillNColsFieldWithValue("S.Anterior", cols[0]+cols[1]) + txarh.fillNColsFieldWithValue(df.format(saldos[0]), cols[2]) + StringUtils.repeat(" ", cols[3]) + txarh.fillNColsFieldWithValue(df.format(saldos[1]), cols[4]) + txarh.fillNColsFieldWithValue(" ", cols[5]) + txarh.fillNColsFieldWithValue(df.format(saldo), cols[6]) + txarh.breakLine();
            //movimientos
            double saldoProv = saldos[0] - saldos[1];
            double saldoAccDebe = saldos[0];
            double saldoAccHaber = saldos[1];

            List<Apunte> apuntes = saldosBean.getApuntes(cc.getCodigo(), cc.getCodigo(), defecha, afecha, ejActivo.getEmpresa());
            Collections.sort(apuntes, txarh.comparatorAps);
            for (int j = 0; j < apuntes.size(); j++) {

                Apunte ap = apuntes.get(j);
                s += txarh.fillNColsFieldWithValue(sdf2.format(ap.getAsiento().getFecha()), cols[0]) +
                        txarh.fillNColsFieldWithValue(ap.getAmpliacion(), cols[1]);
                if (ap.getDebeHaber()){
                    saldoProv += ap.getImporte();
                    saldoAccDebe += ap.getImporte();
                    s += txarh.fillNColsFieldWithValue(df.format(ap.getImporte()),cols[2])
                            + StringUtils.repeat(" ", cols[3])
                            + txarh.fillNColsFieldWithValue("0.00", cols[4])
                            + StringUtils.repeat(" ", cols[5])
                            + txarh.fillNColsFieldWithValue(getTxtSaldoProv(saldoProv, df), cols[6])
                            + StringUtils.repeat(" ", cols[7])
                            + txarh.fillNColsFieldWithValue(ap.getAsiento().getNumAsiento().toString() + "/" + ap.getNumApunte(), cols[8])
                            + txarh.breakLine();

                }else{
                    saldoProv -= ap.getImporte();
                    saldoAccHaber += ap.getImporte();
                    s += txarh.fillNColsFieldWithValue("0.00",cols[2])
                            + StringUtils.repeat(" ", cols[3])
                            + txarh.fillNColsFieldWithValue(df.format(ap.getImporte()), cols[4])
                            + StringUtils.repeat(" ", cols[5])
                            + txarh.fillNColsFieldWithValue(getTxtSaldoProv(saldoProv, df), cols[6])
                            + StringUtils.repeat(" ", cols[7])
                            + txarh.fillNColsFieldWithValue(ap.getAsiento().getNumAsiento().toString() + "/" + ap.getNumApunte(), cols[8])
                            + txarh.breakLine();

                }

            }
            s += txarh.fillNColsFieldWithValue(StringUtils.repeat("-", cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]), cols[0]+cols[1]+cols[2]+cols[3]+cols[4]+cols[5]+cols[6]+cols[7]+cols[8]) + txarh.breakLine();
            s += txarh.fillNColsFieldWithValue("Totales", cols[0]+cols[1]) +
                    txarh.fillNColsFieldWithValue(df.format(saldoAccDebe), cols[2]) +
                    txarh.fillNColsFieldWithValue(" ", cols[3]) +
                    txarh.fillNColsFieldWithValue(df.format(saldoAccHaber), cols[4]) +
                    txarh.fillNColsFieldWithValue(" ", cols[5]) +
                    txarh.fillNColsFieldWithValue(df.format(saldoProv), cols[6]) +
                    txarh.breakLine();

        }


        return s;
    }

    private String getTxtSaldoProv(double saldoProv, DecimalFormat df){
        if (saldoProv>=0.0){
            return df.format(Math.abs(saldoProv)) + " D ";
        }else{
            return df.format(Math.abs(saldoProv)) + " H ";
        }
    }

    public static String getCaption(CuentaContable cc){
        int y = 2;
        return cc.getCodigo() + " " + cc.getNombre();
    }

    public List<CuentaContable> getCuentasContables(){
        String eql = "select cc from contabi_CuentaContable cc join cc.planContable pc where " +
                "pc.id = :pcid order by cc.codigo asc";
        List l = dataManager.load(CuentaContable.class).query(eql)
                .parameter("pcid", ((User) currentAuthentication.getUser()).getEjercicioActivo().getPlanContable().getId() )
                .list();
        return l;
    }



}