package contabilidad.screen.gestioncentrocoste;

import contabilidad.entity.CentroDeCoste;
import contabilidad.entity.Ejercicio;
import contabilidad.entity.PlanContable;
import contabilidad.entity.User;
import contabilidad.screen.centrodecoste.CentroDeCosteEdit;
import io.jmix.core.DataManager;
import io.jmix.core.LoadContext;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.EntityPicker;
import io.jmix.ui.component.Tree;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;

@UiController("contabi_GestionCentroCoste")
@UiDescriptor("gestion-centro-coste.xml")
public class GestionCentroCoste extends Screen {
    @Autowired
    private Button btnCrear;
    @Autowired
    private Button btnRenombrarCentro;
    @Autowired
    private EntityPicker<PlanContable> pkrPlanContable;
    @Autowired
    private Tree<CentroDeCoste> treeCentrosDeCoste;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private CollectionContainer<CentroDeCoste> centroCosteDc;
    @Autowired
    private CollectionLoader<CentroDeCoste> centroCosteDl;
    @Autowired
    private Notifications notifications;
    @Autowired
    private ScreenBuilders screenBuilders;
    @Autowired
    private Button btnExpandir;
    @Autowired
    private Button btnColapsar;
    @Autowired
    private Button btnBorrar;

    private void cargaCentrosDeCosteParaPlan(PlanContable pc) throws Exception{
        CentroDeCoste root = pc.getCentroDeCosteRoot();
        if (root == null){
            root = dataManager.create(CentroDeCoste.class);
            root.setNombre("Root");
            root.setDescripcion("Root");
            root.setPlanContable(pc);
            pc.setCentroDeCosteRoot(root);
            root = dataManager.save(root);
            pc = dataManager.save(pc);

        }
        root = dataManager.load(CentroDeCoste.class).id(root.getId()).fetchPlan("fp-centro-de-coste-recursivo").one();


        centroCosteDl.load();

        treeCentrosDeCoste.expandTree();
    }

    @Subscribe
    public void onAfterInit(AfterInitEvent event) {
        User u = (User) currentAuthentication.getUser();
        PlanContable pc = u.getEjercicioActivo().getPlanContable();
        pkrPlanContable.addValueChangeListener(e->{
            PlanContable pc_ = e.getValue();
            try {
                cargaCentrosDeCosteParaPlan(pc_);
            } catch (Exception ex) {
                notifications.create().withDescription(ex.getMessage()).show();
            }
        });
        btnCrear.addClickListener(e->{
            CentroDeCoste antecesor = treeCentrosDeCoste.getSingleSelected();
            if (antecesor==null){
                notifications.create().withCaption("Seleccionar nodo antecesor").show();
                return;
            }
            StandardEditor<CentroDeCoste> se = (StandardEditor<CentroDeCoste>) screenBuilders.editor(CentroDeCoste.class, this).newEntity()
                    .withOpenMode(OpenMode.DIALOG).build();
            se.addAfterShowListener(ev->{
                CentroDeCosteEdit cce = (CentroDeCosteEdit) se;
                cce.getCentroDeCosteDc().getItem().setCentroDeCosteAntecesor(antecesor);
            });
            se.addAfterCloseListener(ev->{
               if (ev.getCloseAction() == StandardEditor.WINDOW_COMMIT_AND_CLOSE_ACTION){
                   try {
                       cargaCentrosDeCosteParaPlan(pkrPlanContable.getValue());
                   } catch (Exception ex) {
                       notifications.create().withDescription(ex.getMessage()).show();
                   }
               }
            });
            se.show();
        });
        pkrPlanContable.setValue(pc);
        centroCosteDl.load();

        btnRenombrarCentro.addClickListener(e->{
            CentroDeCoste cc = treeCentrosDeCoste.getSingleSelected();
            if (cc==null){
                notifications.create().withDescription("Seleccionar un nodo").show();
                return;
            }else{
                StandardEditor se = (StandardEditor) screenBuilders.editor(CentroDeCoste.class, this).editEntity(cc).withOpenMode(OpenMode.DIALOG).build();
                se.addAfterCloseListener(ev->{
                    centroCosteDl.load();
                });
                se.show();
            }
        });

        btnBorrar.addClickListener(e->{
            notifications.create().withCaption("Funcionalidad pendiente de implementar").show();
        });

        btnExpandir.addClickListener(e->{
            if (treeCentrosDeCoste.getSingleSelected()!=null){
                treeCentrosDeCoste.expand(treeCentrosDeCoste.getSingleSelected());
            }else{
                treeCentrosDeCoste.expandTree();
            }

        });
        btnColapsar.addClickListener(e->{
            if (treeCentrosDeCoste.getSingleSelected()!=null){
                treeCentrosDeCoste.collapse(treeCentrosDeCoste.getSingleSelected());
            }else{
                treeCentrosDeCoste.collapseTree();
            }
        });

        treeCentrosDeCoste.expandTree();
    }

    @Install(to = "centroCosteDl", target = Target.DATA_LOADER)
    private List<CentroDeCoste> centroCosteDlLoadDelegate(LoadContext<CentroDeCoste> loadContext) {
        ArrayList<CentroDeCoste> al =  new ArrayList<CentroDeCoste>();
        CentroDeCoste root = pkrPlanContable.getValue().getCentroDeCosteRoot();
        root = dataManager.load(CentroDeCoste.class).id(root.getId()).fetchPlan("fp-centro-de-coste-recursivo").one();
        al.add(pkrPlanContable.getValue().getCentroDeCosteRoot());
        Iterator<CentroDeCoste> descendientes = new ArrayList(root.getDescendientes()).listIterator();
        //rellenando los descendientes
        List<Iterator> iterators = new ArrayList<Iterator>();
        a:
        while(true){
            while(descendientes.hasNext()){
                CentroDeCoste cc = descendientes.next();

                al.add(cc);
                cc = dataManager.load(CentroDeCoste.class).id(cc.getId()).fetchPlan("fp-centro-de-coste-recursivo").one();
                if (cc.getDescendientes().size()>0){
                    iterators.add(descendientes);
                    descendientes = cc.getDescendientes().listIterator();
                }

            }
            if (iterators.size()>0) {
                descendientes = iterators.get(iterators.size() - 1);
                iterators.remove(descendientes);
            }else{
                break a;
            }

        }
        Collections.sort(al, new Comparator<CentroDeCoste>() {
            @Override
            public int compare(CentroDeCoste o1, CentroDeCoste o2) {
                return o1.getNombre().compareTo(o2.getNombre());
            }
        });
        return al;
    }









}