package contabilidad.screen.informecentrocoste;

import contabilidad.entity.CentroDeCoste;
import contabilidad.entity.ImputacionCentroDeCoste;
import contabilidad.entity.ItemInformeCentroDeCoste;
import contabilidad.entity.PlanContable;
import contabilidad.screen.centrodecoste.SeleccionCentroCoste;
import io.jmix.core.DataManager;
import io.jmix.core.LoadContext;
import io.jmix.ui.ScreenBuilders;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.DateField;
import io.jmix.ui.component.EntityPicker;
import io.jmix.ui.component.VBoxLayout;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;

@UiController("contabi_InformeCentroCoste")
@UiDescriptor("informe-centro-coste.xml")
public class InformeCentroCoste extends Screen {

    CentroDeCoste ccSeleccionado = null;

    @Autowired
    private CollectionContainer<ItemInformeCentroDeCoste> itemsDc;
    @Autowired
    private CollectionLoader<ItemInformeCentroDeCoste> itemsDl;
    @Autowired
    private Button btnCargar;
    @Autowired
    private DateField fechaDesdeField;
    @Autowired
    private DateField fechaHastaField;
    @Autowired
    private VBoxLayout main;
    @Autowired
    private Button selectorCentroCoste;
    @Autowired
    private ScreenBuilders screenBuilders;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private EntityPicker<PlanContable> pkrPlanContable;

    @Subscribe("selectorCentroCoste")
    public void onSelectorCentroCosteClick(Button.ClickEvent event) {
        SeleccionCentroCoste s = (SeleccionCentroCoste) screenBuilders.screen(this).withScreenId("contabi_SeleccionCentroCoste").withOpenMode(OpenMode.DIALOG)
                .build();
        s.addAfterShowListener(e->{
            s.setPlanContable(pkrPlanContable.getValue());
        });
        s.addAfterCloseListener(e->{
            CentroDeCoste cc = s.getSeleccion();
            if (cc!=null){
                this.ccSeleccionado = cc;
                itemsDl.load();
            }
        });
        s.show();
    }



    @Install(to = "itemsDl", target = Target.DATA_LOADER)
    private List<ItemInformeCentroDeCoste> itemsDlLoadDelegate(LoadContext<ItemInformeCentroDeCoste> loadContext) {
        ArrayList<ItemInformeCentroDeCoste> items = new ArrayList<>();
        ArrayList centrosDeCoste = new ArrayList();
        ArrayList<Iterator> iterators = new ArrayList<Iterator>();
        Iterator<CentroDeCoste> currIterator = Arrays.asList(new CentroDeCoste[]{this.ccSeleccionado}).iterator();
        //seleccion de todos los centros de coste que desciendan del seleccionado
        while(true){
            while(currIterator.hasNext()){
                CentroDeCoste ccc = currIterator.next();
                //ccc.setCentroDeCosteAntecesor(antecesors.get(currIterator));
                centrosDeCoste.add(ccc);
                String eql = "select cc from contabi_CentroDeCoste cc where cc.centroDeCosteAntecesor.id = :ccaid";
                List<CentroDeCoste> ccs = dataManager.load(CentroDeCoste.class).query(eql).parameter("ccaid", ccc.getId())
                        .fetchPlan("fp-centro-de-coste-recursivo").list();
                if (ccs.size()>0){
                    iterators.add(currIterator);
                    currIterator = ccs.iterator();

                }
            }
            if (iterators.size()>0){
                currIterator = iterators.get(iterators.size()-1);
                iterators.remove(currIterator);
            }else{
                break;
            }

        }
        //seleccion de todas las imputaciones de los centros de coste incluidos que recaigan en asientos cuya fecha
        //esté dentro del rango fechadesde fechahasta
        String eql = "select icc from contabi_ImputacionCentroDeCoste icc join icc.asiento a join a.ejercicio ej join ej.planContable pc " +
                "where a.fecha >= :fechadesde and a.fecha <= :fechahasta and pc.id = :pcid";
        List<ImputacionCentroDeCoste> iccs = dataManager.load(ImputacionCentroDeCoste.class).query(eql)
                .parameter("fechadesde", fechaDesdeField.getValue()).parameter("fechahasta", fechaHastaField.getValue()).parameter("pcid",pkrPlanContable.getValue().getId())
                .list();
        for (int i = 0; i < iccs.size(); i++) {
            ImputacionCentroDeCoste icc = iccs.get(i);
            ItemInformeCentroDeCoste iicc = dataManager.create(ItemInformeCentroDeCoste.class);
            iicc.setImporte(icc.getImporteConSigno());
            ArrayList<CentroDeCoste> acc = new ArrayList();
            acc.add(icc.getCentroDeCoste());
            CentroDeCoste ccantecesor = null;
            CentroDeCoste currcc = icc.getCentroDeCoste();
            while((ccantecesor = getAntecesorFromBBDD(currcc))!=null){
                acc.add(ccantecesor);
                currcc = ccantecesor;
            }
            Collections.reverse(acc);
            for (int j = 0; j < acc.size(); j++) {
                CentroDeCoste cc = acc.get(j);
                if (j==0){
                    iicc.setNivel1(cc.getNombre());
                }
                if (j==1){
                    iicc.setNivel2(cc.getNombre());
                }
                if (j==2){
                    iicc.setNivel3(cc.getNombre());
                }
                if (j==3){
                    iicc.setNivel4(cc.getNombre());
                }
                if (j==4){
                    iicc.setNivel5(cc.getNombre());
                }
            }
            items.add(iicc);
        }

    return items;
    }

    public CentroDeCoste getAntecesorFromBBDD(CentroDeCoste cc){
        String eql = "select cc.centroDeCosteAntecesor from contabi_CentroDeCoste cc where cc.id = :ccid";
        List<CentroDeCoste> ccl = dataManager.load(CentroDeCoste.class).query(eql).parameter("ccid", cc.getId()).list();
        if (ccl.size()==1){
            return ccl.get(0);
        }else{
            return null;
        }
    }



}