package contabilidad.screen.volcadosaldosiniciales;

import contabilidad.beans.SaldosBean;
import contabilidad.entity.Ejercicio;
import contabilidad.entity.PlanContable;
import contabilidad.entity.SaldoCuentaEjercicio;
import io.jmix.core.DataManager;
import io.jmix.core.LoadContext;
import io.jmix.ui.Dialogs;
import io.jmix.ui.Notifications;
import io.jmix.ui.action.DialogAction;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.DataGrid;
import io.jmix.ui.component.EntityPicker;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

@UiController("contabi_VolcadoSaldosIniciales")
@UiDescriptor("volcado-saldos-iniciales.xml")
public class VolcadoSaldosIniciales extends Screen {

    @Autowired
    private EntityPicker<Ejercicio> pkrEjercicioA;
    @Autowired
    private CollectionContainer<SaldoCuentaEjercicio> saldoCuentaEjerciciosDc;
    @Autowired
    private CollectionLoader<SaldoCuentaEjercicio> saldoCuentaEjerciciosDl;
    @Autowired
    private Button btnCargarSaldos;
    @Autowired
    private Button btnCerrar;
    @Autowired
    private Button btnRealizarVolcado;
    @Autowired
    private DataGrid<SaldoCuentaEjercicio> saldoCuentaEjerciciosTable;
    @Autowired
    private SaldosBean saldosBean;
    @Autowired
    private EntityPicker<Ejercicio> pkrEjercicioDe;
    @Autowired
    private Notifications notifications;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private Dialogs dialogs;

    @Subscribe("btnCargarSaldos")
    public void onBtnCargarSaldosClick(Button.ClickEvent event) {
        try {
            if ((pkrEjercicioDe.getValue()==null)||(pkrEjercicioA.getValue()==null)){
                notifications.create().withDescription("Seleccionar Ejercicios").show();
            }
            Ejercicio ejDe = pkrEjercicioDe.getValue();
            Ejercicio ejA = pkrEjercicioA.getValue();
            if (ejA.getAnno()!=(ejDe.getAnno()+1)){
                notifications.create().withDescription("Debe seleccionar dos ejercicios consecutivos").show();
                return;
            }
            saldoCuentaEjerciciosDl.load();
        }catch(Exception exc){
            notifications.create().withDescription("Error al calcular saldos iniciales").show();
        }
    }

    @Subscribe("btnRealizarVolcado")
    public void onBtnRealizarVolcadoClick(Button.ClickEvent event) {
        dialogs.createOptionDialog().withCaption("Volcado de Saldos")
                .withMessage("El volcado de saldos implica la eliminación de cualquier saldo inicial " +
                        "establecido anteriormente. ¿Desea continuar?")
                .withActions(
                        new DialogAction(DialogAction.Type.YES).withHandler(e->{
                            //eliminacion
                            boolean res = false;
                            try {
                                res = saldosBean.realizaEliminacionDeSaldosCuentaEjercicioParaEjercicio(pkrEjercicioA.getValue());
                            } catch (Exception ex) {
                                notifications.create().withCaption("Error")
                                        .withDescription("Error al borrar los SCE's anteriores: " + ex.getMessage())
                                        .show();
                                return;
                            }
                            if (!res){
                                return;
                            }
                            List l = saldoCuentaEjerciciosDc.getItems();
                            ArrayList al = new ArrayList(l);
                            for (int i = 0; i < al.size(); i++) {
                                SaldoCuentaEjercicio sce = (SaldoCuentaEjercicio) l.get(i);

                                if (sce.getCuentaContable().getCodigo().substring(0,1).compareTo("6")==0){
                                    sce.setSaldoInicialAcreedor(0.0);
                                    sce.setSaldoInicialDeudor(0.0);
                                }
                                if (sce.getCuentaContable().getCodigo().substring(0,1).compareTo("7")==0){
                                    sce.setSaldoInicialAcreedor(0.0);
                                    sce.setSaldoInicialDeudor(0.0);
                                }
                            }
                            for (int i = 0; i < al.size(); i++) {
                                SaldoCuentaEjercicio sce = (SaldoCuentaEjercicio) al.get(i);
                                dataManager.save(sce);
                            }
                            notifications.create()
                                    .withCaption("Saldos iniciales actualizados para el ejerciccio " + pkrEjercicioA.getValue().getDescriptivo())
                                    .show();
                            this.closeWithDefaultAction();
                        }),
                        new DialogAction(DialogAction.Type.NO).withHandler(e->{
                            this.closeWithDefaultAction();
                        })
                ).show();

    }

    @Subscribe("btnCerrar")
    public void onBtnCerrarClick(Button.ClickEvent event) {
        this.closeWithDefaultAction();
    }



    @Install(to = "saldoCuentaEjerciciosDl", target = Target.DATA_LOADER)
    private List<SaldoCuentaEjercicio> saldoCuentaEjerciciosDlLoadDelegate(LoadContext<SaldoCuentaEjercicio> loadContext) {
        List<SaldoCuentaEjercicio> ssccee = new ArrayList<>();
        try {
            ssccee = saldosBean.calculaSaldosCuentaEjercicioPara(pkrEjercicioA.getValue());
            Collections.sort(ssccee, new Comparator<SaldoCuentaEjercicio>() {
                @Override
                public int compare(SaldoCuentaEjercicio o1, SaldoCuentaEjercicio o2) {
                    return o1.getCuentaContable().getCodigo().compareTo(o2.getCuentaContable().getCodigo());
                }
            });
        } catch (Exception e) {
            notifications.create().withDescription("Error al calcular saldos iniciales").show();
        }
        return ssccee;
    }


    public void percargaInformacionParaEjercicio(Ejercicio ej){
        PlanContable pc = ej.getPlanContable();
        String eql = "select ej from contabi_Ejercicio ej where ej.anno = :anno";
        try {
            Ejercicio ejSiguiente = dataManager.load(Ejercicio.class).query(eql)
                    .parameter("anno", ej.getAnno() + 1).one();
            pkrEjercicioDe.setValue(ej);
            pkrEjercicioA.setValue(ejSiguiente);
            this.onBtnCargarSaldosClick(null);
        }catch(Exception exc){
            notifications.create().withDescription(exc.getMessage()).show();
        }
    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        int y = 2;
    }







}